from __future__ import annotations

from media_platform.lang.serialization import Deserializable


class AudioStream(Deserializable):
    def __init__(self, index: int, duration: int, bitrate: int, codec_tag: str, codec_name: str, codec_long_name: str,
                 sample_rate: int, channels: int, channel_layout: str):
        self.index = int(index)
        self.duration = duration
        self.bitrate = int(bitrate) if bitrate else None
        self.codec_tag = codec_tag
        self.codec_name = codec_name
        self.codec_long_name = codec_long_name
        self.sample_rate = int(sample_rate) if sample_rate else None
        self.channel_layout = channel_layout
        self.channels = int(channels) if channels else None

    @classmethod
    def deserialize(cls, data: dict) -> AudioStream:
        return AudioStream(data.get('index'), data.get('duration'), data.get('bitrate'),
                           data.get('codecTag'), data.get('codecName'), data.get('codecLongName'),
                           data.get('sampleRate'), data.get('channels'), data.get('channelLayout'))

    def serialize(self) -> dict:
        return {
            'index': self.index,
            'duration': self.duration,
            'bitrate': self.bitrate,
            'codecTag': self.codec_tag,
            'codecName': self.codec_name,
            'codecLongName': self.codec_long_name,
            'sampleRate': self.sample_rate,
            'channelLayout': self.channel_layout,
            'channels': self.channels
        }



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.metadata.audio.audio_stream import AudioStream
from media_platform.metadata.video.transparency import Transparency
from media_platform.metadata.video.video_format import VideoFormat
from media_platform.metadata.video.video_stream import VideoStream


class VideoBasic(Serializable, Deserializable):
    def __init__(self, video_streams: [VideoStream], audio_streams: [AudioStream], video_format: VideoFormat = None,
                 video_interlaced: bool = False, video_tbr: dict = None, transparency: Transparency = None):
        self.video_streams = video_streams or []
        self.audio_streams = audio_streams or []
        self.video_format = video_format
        self.video_interlaced = video_interlaced
        self.video_tbr = video_tbr
        self.transparency = transparency

    @classmethod
    def deserialize(cls, data: dict) -> VideoBasic:
        video_streams = [VideoStream.deserialize(stream) for stream in data['videoStreams']]
        audio_streams = [AudioStream.deserialize(stream) for stream in data['audioStreams']]
        video_format = VideoFormat.deserialize(data['format']) if data.get('format') else None

        return VideoBasic(video_streams, audio_streams, video_format, data.get('interlaced'),
                          data.get('tbr'), data.get('transparency'))

    def serialize(self):
        return {
            'videoStreams': [video_stream.serialize() for video_stream in self.video_streams],
            'audioStreams': [audio_stream.serialize() for audio_stream in self.audio_streams],
            'format': self.video_format.serialize() if self.video_format else None,
            'interlaced': self.video_interlaced,
            'tbr': self.video_tbr,
            'transparency': self.transparency
        }



from __future__ import annotations

from media_platform.auth.token import Token

_VERB = 'urn:service:image.operations'


class Gravity:
    center = 'center'
    north = "north"
    north_west = "north-west"
    west = "west"
    south_west = "south-west"
    south = "south"
    south_east = "south-east"
    east = "east"
    north_east = "north-east"


class Watermark:
    def __init__(self, path: str, opacity: int = 50, proportions: float = 0.25, gravity: Gravity = Gravity.center):
        self.path = path
        self.opacity = opacity
        self.proportions = proportions
        self.gravity = gravity

    def to_claim(self) -> dict:
        watermark = {}
        if self.path:
            watermark['path'] = self.path
        if self.opacity:
            watermark['opacity'] = self.opacity
        if self.proportions:
            watermark['proportions'] = self.proportions
        if self.gravity:
            watermark['gravity'] = self.gravity

        return {
            'wmk': watermark
        }


class Policy:
    def __init__(self, path: str, max_width: int = None, max_height: int = None, min_blur: float = None):
        self.path = path
        self.max_width = max_width
        self.max_height = max_height
        self.min_blur = min_blur

    def to_claim(self) -> dict:
        policy = {}
        if self.path:
            policy['path'] = self.path
        if self.max_height:
            policy['height'] = '<=%s' % self.max_height
        if self.max_width:
            policy['width'] = '<=%s' % self.max_width
        if self.min_blur:
            policy['blur'] = '>=%s' % self.min_blur

        return {
            'obj': [[policy]]
        }


class ImageToken(Token):
    def __init__(self, issuer: str, subject: str, policy: Policy = None, watermark: Watermark = None,
                 issued_at: int = None, expiration: int = None, additional_claims: dict = None, token_id: str = None):
        super().__init__(issuer, subject, [_VERB], issued_at, expiration, additional_claims, token_id)
        self.policy = policy
        self.watermark = watermark

    def to_claims(self) -> dict:
        claims = super().to_claims()

        if self.watermark:
            claims.update(self.watermark.to_claim())
        if self.policy:
            claims.update(self.policy.to_claim())

        return claims

    @classmethod
    def from_token(cls, token: Token) -> ImageToken:
        return cls(token.issuer, token.subject, None, None, token.issued_at, token.expiration, token.additional_claims,
                   token.id)



from typing import Dict

from media_platform.lang.serialization import Deserializable, Serializable
from media_platform.service.live_service.geo_location import GeoLocation
from media_platform.service.live_service.stream_protocol import StreamProtocol


class StreamPublishEndpoint(Serializable, Deserializable):
    def __init__(self, url: str, protocol: StreamProtocol, geo: GeoLocation = None):
        self.url = url
        self.protocol = protocol
        self.geo = geo

    def serialize(self) -> Dict:
        return {
            'url': self.url,
            'protocol': self.protocol,
            'geo': self.geo.serialize() if self.geo else None
        }

    @classmethod
    def deserialize(cls, data: dict):
        geo = GeoLocation.deserialize(data['geo']) if data.get('geo') else None

        return cls(data['url'], data['protocol'], geo)



from __future__ import annotations

from media_platform.error_code import ErrorCode
from media_platform.job.result.job_result import JobResult


class JobCallbackFailedResult(JobResult):
    type = None

    def __init__(self, message: str = None, job_result: dict = None):
        super().__init__(ErrorCode.job_callback_failed, message)
        self.job_result = job_result

    @classmethod
    def deserialize(cls, data: dict or None) -> JobCallbackFailedResult or None:
        if not data:
            return None

        result = JobResult.deserialize(data)
        result.__class__ = JobCallbackFailedResult
        payload = data.get('payload') or {}
        result.job_result = payload.get('jobResult')

        return result

    def serialize(self) -> dict:
        data = super().serialize()
        data['payload'] = {
            'jobResult': self.job_result
        }

        return data



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.metadata.audio_file_metadata import AudioFileMetadata
from media_platform.service.media_platform_request import MediaPlatformRequest


class ExtractMetadataRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'GET', base_url + '/audio/metadata', AudioFileMetadata)
        self.path = None

    def set_path(self, path: str) -> ExtractMetadataRequest:
        self.path = path
        return self

    def execute(self) -> AudioFileMetadata:
        return super().execute()

    def _params(self) -> dict:
        return {
            'path': self.path
        }



from __future__ import annotations

from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.job.job import Job


class SubsetFontSpecification(Specification):
    def __init__(self, destination: Destination, language_code: str = None):
        self.destination = destination
        self.language_code = language_code

    @classmethod
    def deserialize(cls, data: dict) -> SubsetFontSpecification:
        destination = Destination.deserialize(data['destination'])

        return SubsetFontSpecification(destination, data.get('languageCode'))

    def serialize(self) -> dict:
        return {
            'destination': self.destination.serialize(),
            'languageCode': self.language_code,
        }


class SubsetFontJob(Job):
    type = 'urn:job:text.font.subset'
    specification_type = SubsetFontSpecification



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient


class MediaPlatformService:
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient):
        self._domain = domain

        self._authenticated_http_client = authenticated_http_client

        self._base_url = 'https://' + domain + '/_api'



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable


class ImageBasic(Serializable, Deserializable):
    def __init__(self, width: int = None, height: int = None, image_format: str = None, color_space: str = None):
        self.width = int(width) if width else None
        self.height = int(height) if height else None
        self.format = image_format
        self.color_space = color_space

    @classmethod
    def deserialize(cls, data: dict) -> ImageBasic:
        return ImageBasic(data['width'], data['height'], data['format'], data.get('colorspace'))

    def serialize(self) -> dict:
        return {
            'width': self.width,
            'height': self.height,
            'format': self.format,
            'colorspace': self.color_space
        }



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable


class JobResult(Serializable, Deserializable):
    type = None

    def __init__(self, code: int = None, message: str = None):
        self.code = code or 0
        self.message = message or 'OK'

    @classmethod
    def deserialize(cls, data: dict or None) -> JobResult or None:
        if not data:
            return None

        return cls(data['code'], data['message'])

    def serialize(self) -> dict:
        return {
            'code': self.code,
            'message': self.message
        }



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.image_operation_job import ImageOperationSpecification
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class ImageOperationRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/images/operations', FileDescriptor)
        self.source = None
        self.specification = None

    def set_source(self, source: Source) -> ImageOperationRequest:
        self.source = source
        return self

    def set_specification(self, specification: ImageOperationSpecification) -> ImageOperationRequest:
        self.specification = specification
        return self

    def _params(self) -> dict:
        return {
            'source': self.source.serialize(),
            'specification': self.specification.serialize(),
        }

    def execute(self) -> FileDescriptor:
        return super().execute()



from __future__ import annotations

from media_platform.job.job import Job
from media_platform.lang.serialization import Deserializable


class JobCallbackPayload(Deserializable):
    def __init__(self, job: Job, attachment: dict = None):
        self.job = job
        self.attachment = attachment

    @classmethod
    def deserialize(cls, data: dict) -> JobCallbackPayload:
        return cls(Job.deserialize(data['job']), data.get('attachment'))



from __future__ import annotations

from media_platform.lang.serialization import Deserializable


class Rectangle(Deserializable):
    def __init__(self, x: int, y: int, width: int, height: int):
        self.x = x
        self.y = y
        self.width = width
        self.height = height

    @classmethod
    def deserialize(cls, data: dict) -> Rectangle:
        return Rectangle(data['x'], data['y'], data['width'], data['height'])



from __future__ import annotations

from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor


class FileUploadCallbackPayload(Deserializable):
    def __init__(self, file_descriptor: FileDescriptor, attachment: dict = None):
        self.file_descriptor = file_descriptor
        self.attachment = attachment

    @classmethod
    def deserialize(cls, data: dict) -> FileUploadCallbackPayload:
        return cls(FileDescriptor.deserialize(data['file']), data.get('attachment'))



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.metadata.file_metadata import FileMetadata
from media_platform.service.media_platform_request import MediaPlatformRequest
# noinspection PyProtectedMember
from media_platform.metadata.file_metadata_deserializer import _FileMetadataDeserializer


class Detection:
    transparency = 'transparency'
    valid_values = [transparency]

    @classmethod
    def validate(cls, value):
        if value not in cls.valid_values:
            raise ValueError('Must be one of: ' + ','.join(cls.valid_values))


class ExtractMetadataRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'GET', base_url + '/files/metadata/extract',
                         _FileMetadataDeserializer)
        self.path = None
        self.detections = []

    def set_path(self, path: str) -> ExtractMetadataRequest:
        self.path = path
        return self

    def set_detections(self, detections: [Detection]) -> ExtractMetadataRequest:
        self.detections = detections
        return self

    def add_detection(self, *detection: [Detection]) -> ExtractMetadataRequest:
        self.detections.extend(detection)
        return self

    def execute(self) -> FileMetadata:
        return super().execute()

    def _params(self) -> dict:
        return {
            'path': self.path,
            'detections': ','.join(self.detections)
        }



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.file_service.create_file_request import CreateFileRequest
from media_platform.service.file_service.create_files_response import _CreateFilesResponse
from media_platform.service.media_platform_request import MediaPlatformRequest


class CreateFilesRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/files', FileDescriptor)
        self.file_requests = []

    def execute(self) -> Deserializable or None:
        # TODO: Replace with single request once server supports that
        file_descriptors = [self.authenticated_http_client.post(self.url, f._params(), self.response_payload_type) for f
                            in self.file_requests]

        return _CreateFilesResponse(file_descriptors)

    def set_file_requests(self, file_requests: [CreateFileRequest]) -> CreateFilesRequest:
        self.file_requests = file_requests
        return self

    def add_file(self, *file_request: [CreateFileRequest]) -> CreateFilesRequest:
        self.file_requests.extend(file_request)
        return self

    def _params(self):
        return {
            'files': [file_request._params() for file_request in self.file_requests]
        }



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.flow_control_service.flow import Flow
from media_platform.service.flow_control_service.flow_state import FlowState
from media_platform.service.flow_control_service.invocation import Invocation
from media_platform.service.media_platform_request import MediaPlatformRequest


class FlowInvocationRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/flow_control/flow', FlowState)
        self.invocation = None
        self.flow = None

    def set_invocation(self, invocation: Invocation) -> FlowInvocationRequest:
        self.invocation = invocation
        return self

    def set_flow(self, flow: Flow) -> FlowInvocationRequest:
        self.flow = flow
        return self

    def validate(self):
        self.flow.validate()
        self._validate_entry_points()

    def execute(self) -> FlowState:
        return super(FlowInvocationRequest, self).execute()

    def _params(self) -> dict:
        return {
            'invocation': self.invocation.serialize(),
            'flow': self.flow.serialize(),
        }

    def _validate_entry_points(self):
        invalid_entry_points = [e for e in self.invocation.entry_points if e not in self.flow.components]
        if invalid_entry_points:
            raise ValueError('entry points not defined in components: %s' % invalid_entry_points)



from __future__ import annotations

from media_platform.exception.media_platform_exception import MediaPlatformException
from media_platform.lang.serialization import Serializable, Deserializable


class RestResult(Serializable, Deserializable):
    def __init__(self, code: int, message: str, payload: dict or list = None):
        self.code = code
        self.message = message
        self.payload = payload

    @classmethod
    def deserialize(cls, data: dict) -> RestResult:
        return RestResult(data['code'], data['message'], data.get('payload'))

    def serialize(self) -> dict:
        return {
            'code': self.code,
            'message': self.message,
            'payload': self.payload
        }

    def raise_for_code(self):
        if self.code != 0:
            # todo: code -> exception mapper
            raise MediaPlatformException()



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.flow_control_service.flow_state import FlowState
from media_platform.service.media_platform_request import MediaPlatformRequest


class FlowStateRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'GET', base_url + '/flow_control/flow/', FlowState)
        self.flow_id = None
        self._url = base_url + '/flow_control/flow/'

    def set_id(self, flow_id: str) -> FlowStateRequest:
        self.flow_id = flow_id
        return self

    def execute(self) -> FlowState:
        self.url = self._url + self.flow_id

        return super().execute()



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.destination import Destination
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class CopyFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/copy/file', FileDescriptor)
        self.source = None
        self.destination = None

    def set_source(self, source: Source) -> CopyFileRequest:
        self.source = source
        return self

    def set_destination(self, destination: Destination) -> CopyFileRequest:
        self.destination = destination
        return self

    def _params(self) -> dict:
        return {
            'source': self.source.serialize(),
            'destination': self.destination.serialize(),
        }



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.extract_archive.extract_archive_job import ExtractArchiveJob
from media_platform.job.extract_archive.extraction_report import ExtractionReport
from media_platform.service.callback import Callback
from media_platform.service.destination import Destination
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class ExtractArchiveRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/archive/extract', ExtractArchiveJob)
        self.source = None
        self.destination = None
        self.report = None
        self.callback = None

    def set_source(self, source: Source) -> ExtractArchiveRequest:
        self.source = source
        return self

    def set_destination(self, destination: Destination) -> ExtractArchiveRequest:
        self.destination = destination
        return self

    def set_report(self, extraction_report: ExtractionReport) -> ExtractArchiveRequest:
        self.report = extraction_report
        return self

    def set_callback(self, callback: Callback) -> ExtractArchiveRequest:
        self.callback = callback
        return self

    def execute(self) -> ExtractArchiveJob:
        return super().execute()

    def _params(self) -> dict:
        return {
            'source': self.source.serialize(),
            'destination': self.destination.serialize(),
            'extractedFilesReport': self.report.serialize() if self.report else None,
            'jobCallback': self.callback.serialize() if self.callback else None
        }



from __future__ import annotations

from media_platform.lang.serialization import Deserializable, Serializable


class VideoFormat(Serializable, Deserializable):
    def __init__(self, long_name: str = None, bitrate: int = None, duration: int = None, size: int = None):
        self.long_name = long_name
        self.duration = duration
        self.bitrate = int(bitrate) if bitrate else None
        self.size = int(size) if size else None

    @classmethod
    def deserialize(cls, data: dict) -> VideoFormat:
        return VideoFormat(data.get('formatLongName'),
                           data.get('bitrate'),
                           data.get('duration'),
                           data.get('size'))

    def serialize(self) -> dict:
        return {
            'formatLongName': self.long_name,
            'duration': self.duration,
            'bitrate': self.bitrate,
            'size': self.size,
        }



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.metadata.image.image_features import ImageFeatures
from media_platform.service.media_platform_request import MediaPlatformRequest


class Feature:
    faces = 'faces'
    labels = 'labels'
    colors = 'colors'
    explicit_content = 'explicit_content'
    crop_hints = 'crop_hints'

    values = [faces, labels, colors, explicit_content, crop_hints]

    @classmethod
    def has_value(cls, value: str) -> bool:
        return value in cls.values


class ExtractFeaturesRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'GET', base_url + '/images/features', ImageFeatures)
        self.path = None
        self.features = []

    def set_path(self, path: str) -> ExtractFeaturesRequest:
        self.path = path
        return self

    def set_features(self, features: [Feature]) -> ExtractFeaturesRequest:
        self.features = features
        return self

    def add_features(self, *features: [Feature]) -> ExtractFeaturesRequest:
        self.features.extend(features)
        return self

    def _params(self) -> dict:
        return {
            'path': self.path,
            'features': ','.join(self.features)
        }

    def execute(self) -> ImageFeatures:
        return super().execute()



from __future__ import annotations

import time
from furl import furl
import requests

from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.auth.token import Token
from media_platform.service.file_service.attachment import Attachment
from media_platform.service.file_service.inline import Inline


class DownloadFileRequest:
    def __init__(self, app_id: str, authenticator: AppAuthenticator, base_url: str):
        self._app_urn = 'urn:app:' + app_id
        self._url = base_url.replace('_api', '').replace('.appspot.com', '.wixmp.com')
        self._authenticator = authenticator

        self.path = None
        self.ttl = 600  # seconds
        self.attachment = None
        self.inline = None
        self.on_expired_redirect_to = None

    def set_path(self, path: str) -> DownloadFileRequest:
        self.path = path
        return self

    def set_ttl(self, ttl: int) -> DownloadFileRequest:
        self.ttl = ttl
        return self

    def set_attachment(self, attachment: Attachment) -> DownloadFileRequest:
        self.attachment = attachment
        return self

    def set_inline(self, inline: Inline) -> DownloadFileRequest:
        self.inline = inline
        return self

    def set_on_expired_redirect_to(self, on_expired_redirect_to: str) -> DownloadFileRequest:
        self.on_expired_redirect_to = on_expired_redirect_to
        return self

    def url(self) -> str:
        additional_claims = {
            'obj': [[{'path': self.path}]]
        }
        if self.on_expired_redirect_to:
            additional_claims['red'] = self.on_expired_redirect_to

        token = Token(
            self._app_urn,
            self._app_urn,
            ['urn:service:file.download'],
            expiration=int(time.time()) + self.ttl,
            additional_claims=additional_claims
        )

        signed_token = self._authenticator.sign_token(token)

        url = furl(self._url).add(path=self.path).add(query_params={'token': signed_token})

        if self.attachment and self.inline:
            raise ValueError('Can\'t set both attachment and inline')

        if self.attachment:
            url.add(query_params={'filename': self.attachment.file_name})
        elif self.inline:
            url.add(query_params={'filename': self.inline.file_name})
            url.add(query_params={'inline': ''})

        return url.url

    def execute(self) -> requests.Response:
        # http://docs.python-requests.org/en/master/user/advanced/#body-content-workflow

        # if you don't close the response, don't come complaining about connection leakage :)
        return requests.get(self.url(), stream=True)



from __future__ import annotations

from media_platform.lang.serialization import Serializable


class ExternalAuthorization(Serializable):
    def __init__(self, headers: dict):
        self.headers = headers

    def serialize(self) -> dict:
        return {
            'headers': self.headers
        }

    @classmethod
    def deserialize(cls, data: dict) -> ExternalAuthorization:
        return cls(data['headers'])



from __future__ import annotations

import logging
from abc import ABC

from media_platform.lang.serialization import Deserializable, Serializable
from media_platform.service.live_service.stream_error_info import StreamErrorInfo


class StreamParamsOutOfRangeErrorInfo(StreamErrorInfo):
    def __init__(self, enforced_stream_params: EnforcedStreamParams, actual_params: Params):
        self.enforced_stream_params = enforced_stream_params
        self.actual_params = actual_params

    def serialize(self) -> dict:
        return {
            'enforcedStreamParams': self.enforced_stream_params.serialize,
            'actualParams': self.actual_params.serialize
        }

    @classmethod
    def deserialize(cls, data: dict) -> StreamParamsOutOfRangeErrorInfo:
        return cls(EnforcedStreamParams.deserialize(data['enforcedStreamParams']),
                   Params.deserialize(data['actualParams']))


class StreamParamsOutOfRange(Exception):
    def __init__(self, message: str, error_info: StreamParamsOutOfRangeErrorInfo):
        super(StreamParamsOutOfRange, self).__init__(message)
        self.error_info = error_info


class Params(Serializable, Deserializable):
    def __init__(self, width: int = None, height: int = None, bitrate: int = None):
        self.width = width
        self.height = height
        self.bitrate = bitrate

    def serialize(self) -> dict:
        data = {}
        if self.width:
            data['width'] = self.width

        if self.height:
            data['height'] = self.height

        if self.bitrate:
            data['bitrate'] = self.bitrate

        return data

    @classmethod
    def deserialize(cls, data: dict) -> Params:
        return cls(data.get('width'), data.get('height'), data.get('bitrate'))

    def __lt__(self, other: Params) -> bool:
        if not isinstance(other, Params):
            return NotImplemented

        if self.width and other.width and self.width < other.width:
            return True

        if self.height and other.height and self.height < other.height:
            return True

        if self.bitrate and other.bitrate and self.bitrate < other.bitrate:
            return True

        return False

    def __eq__(self, other: Params) -> bool:
        if not isinstance(other, Params):
            return NotImplemented

        return self.width == other.width and self.height == other.height and self.bitrate == other.bitrate

    def __repr__(self) -> str:
        return str(self.serialize())


class Enforceable(Serializable, ABC):
    name = None

    def is_valid(self, params: Params) -> bool:
        raise NotImplementedError()

    def __repr__(self) -> str:
        return str(self.serialize())


class ParamsRange(Enforceable):
    name = 'range'

    def __init__(self, min_values: [Params] = None, max_values: [Params] = None):
        self.min_values = min_values
        self.max_values = max_values

    def is_valid(self, params: Params) -> bool:
        if self.min_values and params < self.min_values:
            return False

        if self.max_values and params > self.max_values:
            return False

        return True

    def serialize(self) -> dict:
        data = {}
        if self.min_values:
            data['minValues'] = self.min_values.serialize()

        if self.max_values:
            data['maxValues'] = self.max_values.serialize()

        return data

    @classmethod
    def deserialize(cls, data: dict) -> ParamsRange:
        min_values_data = data.get('minValues')
        max_values_data = data.get('maxValues')
        min_values = Params.deserialize(min_values_data) if min_values_data else None
        max_values = Params.deserialize(max_values_data) if max_values_data else None

        return cls(min_values, max_values)


class ParamsOptions(Enforceable):
    name = 'options'

    def __init__(self, options: [Params]):
        self.options = options

    def is_valid(self, params: Params) -> bool:
        return params in self.options

    def serialize(self) -> list:
        return [o.serialize() for o in self.options]

    @classmethod
    def deserialize(cls, data: list) -> ParamsOptions:
        return cls([Params.deserialize(d) for d in data])


class EnforcedStreamParams(Serializable):
    def __init__(self, params_range: ParamsRange = None, params_options: ParamsOptions = None):
        if params_range and params_options:
            raise ValueError('Either range or options must be specified, not both')

        self.params_range = params_range
        self.params_options = params_options

    def enforce(self, params: Params):
        transposed_params = Params(params.height, params.width, params.bitrate)

        for enforceable in [self.params_range, self.params_options]:
            if enforceable:
                if not enforceable.is_valid(params) and not enforceable.is_valid(transposed_params):
                    error_info = StreamParamsOutOfRangeErrorInfo(self, params)
                    raise StreamParamsOutOfRange('%s not in %s %s' % (params, enforceable.name, enforceable),
                                                 error_info)

                logging.debug('%s satisfies enforcement %s: %s' % (params, enforceable.name, enforceable))

    def serialize(self) -> dict:
        data = {}
        if self.params_range:
            data['paramsRange'] = self.params_range.serialize()

        if self.params_options:
            data['paramsOptions'] = self.params_options.serialize()

        return data

    @classmethod
    def deserialize(cls, data: dict) -> EnforcedStreamParams:
        data = data or {}
        params_range_data = data.get('paramsRange')
        params_options_data = data.get('paramsOptions')
        params_range = ParamsRange.deserialize(params_range_data) if params_range_data else None
        params_options = ParamsOptions.deserialize(params_options_data) if params_options_data else None

        return cls(params_range, params_options)

    def __repr__(self) -> str:
        return str(self.serialize())



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.metadata.audio.audio_format import AudioFormat
from media_platform.metadata.audio.audio_stream import AudioStream


class AudioBasic(Serializable, Deserializable):
    def __init__(self, audio_streams: [AudioStream], audio_format: AudioFormat = None):
        self.audio_streams = audio_streams or []
        self.audio_format = audio_format

    @classmethod
    def deserialize(cls, data: dict) -> AudioBasic:
        audio_streams = [AudioStream.deserialize(stream) for stream in data['audioStreams']]
        audio_format = AudioFormat.deserialize(data['format']) if data.get('format') else None

        return AudioBasic(audio_streams, audio_format)

    def serialize(self) -> dict:
        return {
            'audioStreams': [audio_stream.serialize() for audio_stream in self.audio_streams],
            'format': self.audio_format.serialize() if self.audio_format else None
        }



from abc import ABC, abstractmethod


class Serializable(ABC):

    @abstractmethod
    def serialize(self) -> dict:
        pass


class Deserializable(ABC):

    @classmethod
    @abstractmethod
    def deserialize(cls, data: dict):
        pass



from __future__ import annotations

from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor


class MediaType:
    undefined = ''
    image = 'image'
    video = 'video'
    audio = 'audio'
    font = 'font'


class FileMetadata(Deserializable):
    def __init__(self, media_type: str, file_descriptor: FileDescriptor, basic: Deserializable = None):
        self.media_type = media_type
        self.file_descriptor = file_descriptor
        self.basic = basic

    @classmethod
    def deserialize(cls, data: dict) -> FileMetadata:
        file_descriptor = FileDescriptor.deserialize(data['fileDescriptor'])

        return FileMetadata(MediaType.undefined, file_descriptor)



from typing import Type

import urllib3
import requests
from requests.utils import default_headers
from requests.adapters import HTTPAdapter
from requests.exceptions import RetryError
from requests.structures import CaseInsensitiveDict
from requests_toolbelt import MultipartEncoder

from media_platform.lang.serialization import Deserializable
from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.exception.media_platform_exception import MediaPlatformException
from media_platform.http_client.response_processor import ResponseProcessor


class AuthenticatedHTTPClient:
    USER_AGENT = 'WixMP Python SDK 2.x'
    APPLICATION_JSON = 'application/json'
    RETRYABLE_CODES = [500, 503, 504, 429]
    RETRYABLE_METHODS = ['GET', 'POST', 'PUT', 'DELETE']
    TIMEOUT = 60

    def __init__(self, app_authenticator: AppAuthenticator, retry_count: int = 5, retry_backoff_factor: float = 0.2):
        self._app_authenticator = app_authenticator
        self._session = requests.Session()

        retry = urllib3.Retry(total=retry_count, backoff_factor=retry_backoff_factor,
                              status_forcelist=self.RETRYABLE_CODES, method_whitelist=self.RETRYABLE_METHODS,
                              raise_on_status=False)

        self._session.mount('http://', HTTPAdapter(max_retries=retry))
        self._session.mount('https://', HTTPAdapter(max_retries=retry))

    def get(self, url: str, params: dict = None, payload_type: Deserializable = None) -> Deserializable or None:
        return self._send_request('GET', url, params=params, payload_type=payload_type)

    def post(self, url: str, data: dict = None, payload_type: Deserializable = None) -> Deserializable or None:
        return self._send_request('POST', url, json=data, payload_type=payload_type)

    def put(self, url: str, data: dict = None, payload_type: Deserializable = None) -> Deserializable or None:
        return self._send_request('PUT', url, json=data, payload_type=payload_type)

    def delete(self, url: str, params: dict = None, payload_type: Deserializable = None) -> Deserializable or None:
        return self._send_request('DELETE', url, params=params, payload_type=payload_type)

    def post_data(self, url: str, content: str, mime_type: str, params: dict = None,
                  payload_type: Type[Deserializable] = None, filename: str = None,
                  response_processor: callable = None) -> Deserializable or None:
        fields = {
            'file': (filename or 'file-name', content, mime_type)
        }
        fields.update(params)

        encoder = MultipartEncoder(fields)

        try:
            headers = self._base_headers()
            headers['Content-Type'] = encoder.content_type
            response = self._session.post(url, data=encoder, headers=headers)
        except RetryError as e:
            raise MediaPlatformException(cause=e)

        if response_processor:
            return response_processor(response)
        else:
            return ResponseProcessor.process(response, payload_type)

    def _send_request(self, verb: str, url: str, json: dict = None, params: dict = None,
                      payload_type: Deserializable = None) -> Deserializable or None:
        try:
            response = self._session.request(verb, url, params=params, json=json, headers=self._headers(),
                                             timeout=self.TIMEOUT)
        except RetryError as e:
            raise MediaPlatformException(cause=e)

        return ResponseProcessor.process(response, payload_type)

    def _headers(self) -> CaseInsensitiveDict:
        headers = self._base_headers()

        signed_token = self._app_authenticator.default_signed_token()
        headers['Authorization'] = signed_token

        return headers

    def _base_headers(self) -> CaseInsensitiveDict:
        headers = default_headers()
        headers['User-Agent'] = self.USER_AGENT
        headers['Accept'] = self.APPLICATION_JSON

        return headers



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.job_service.job_list import JobList
from media_platform.service.list_request import _ListRequest


class JobListRequest(_ListRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, base_url + '/jobs', JobList)
        self.issuer = None
        self.type = None
        self.status = None
        self.group_id = None
        self.file_id = None
        self.path = None

    def set_issuer(self, issuer: str) -> JobListRequest:
        self.issuer = issuer
        return self

    def set_type(self, job_type: str) -> JobListRequest:
        self.type = job_type
        return self

    def set_status(self, status: str) -> JobListRequest:
        self.status = status
        return self

    def set_path(self, path: str) -> JobListRequest:
        self.path = path
        return self

    def set_group_id(self, group_id: str) -> JobListRequest:
        self.group_id = group_id
        return self

    def set_file_id(self, file_id: str) -> JobListRequest:
        self.file_id = file_id
        return self

    def execute(self) -> JobList:
        return super().execute()

    def _params(self) -> dict:
        params = super()._params()

        if self.issuer:
            params['issuer'] = self.issuer

        if self.type:
            params['type'] = self.type

        if self.status:
            params['status'] = self.status

        if self.group_id:
            params['groupId'] = self.group_id

        if self.file_id:
            params['fileId'] = self.file_id

        if self.path:
            params['path'] = self.path

        return params



from __future__ import annotations

from typing import cast

from media_platform.job.job_group import JobGroup


class ConvertFontJobGroup(JobGroup):

    @classmethod
    def deserialize(cls, data: dict) -> ConvertFontJobGroup:
        return cast(ConvertFontJobGroup, super().deserialize(data))



from __future__ import annotations

from media_platform.lang.serialization import Deserializable


class ExplicitContent(Deserializable):
    def __init__(self, name: str, likelihood: str):
        self.name = name
        self.likelihood = likelihood

    @classmethod
    def deserialize(cls, data: dict) -> ExplicitContent:
        return ExplicitContent(data['name'], data['likelihood'])



from __future__ import annotations

from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor


class FileList(Deserializable):
    def __init__(self, next_page_token: str, files: [FileDescriptor]):
        self.next_page_token = next_page_token
        self.files = files

    @classmethod
    def deserialize(cls, data: dict) -> FileList:
        files = [FileDescriptor.deserialize(f) for f in data['files']]

        return FileList(data['nextPageToken'], files)



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_request import MediaPlatformRequest


class CloseStreamRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'DELETE', base_url + '/live/streams/', None)

        self.stream_id = None
        self._url = base_url + '/live/streams/'

    def set_stream_id(self, stream_id: str) -> CloseStreamRequest:
        self.stream_id = stream_id
        return self

    def execute(self):
        self.url = self._url + self.stream_id
        return super().execute()



from __future__ import annotations

from media_platform.lang.serialization import Deserializable, Serializable
from media_platform.service.flow_control_service.flow_error import FlowError
from media_platform.service.flow_control_service.invocation import Invocation
from media_platform.service.flow_control_service.operation import Operation


class FlowStatus:
    idle = 'idle'
    working = 'working'
    success = 'success'
    error = 'error'
    aborted = 'aborted'


class FlowState(Deserializable, Serializable):
    def __init__(self, flow_id: str, status: FlowStatus, invocation: Invocation, operations: [str, Operation],
                 flow_error: FlowError = None):
        self.id = flow_id
        self.invocation = invocation
        self.operations = operations
        self.status = status
        self.flow_error = flow_error

    @classmethod
    def deserialize(cls, data: dict) -> FlowState:
        invocation = Invocation.deserialize(data['invocation'])
        operations_data = data.get('operations', {})
        operations = {k: Operation.deserialize(v) for k, v in operations_data.items()}
        flow_error_data = data.get('error')
        flow_error = FlowError.deserialize(flow_error_data) if flow_error_data else None

        return FlowState(data['id'], data['status'], invocation, operations, flow_error)

    def serialize(self) -> dict:
        return {
            'id': self.id,
            'invocation': self.invocation.serialize(),
            'operations': {k: v.serialize() for k, v in self.operations.items()},
            'status': self.status,
            'error': self.flow_error.serialize() if self.flow_error else None
        }



from __future__ import annotations

from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification
from media_platform.job.job import Job


class ScanFileSpecification(Specification):
    def serialize(self) -> dict:
        return {}

    @classmethod
    def deserialize(cls, data: dict) -> ScanFileSpecification:
        return cls()


class ScanFileJob(Job):
    type = JobType.av_scan
    specification_type = ScanFileSpecification



from __future__ import annotations

from typing import Type

from media_platform.service.source import Source
from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.replace_extra_metadata_job import ReplaceAudioExtraMetadataSpecification, \
    ReplaceExtraMetadataJob
from media_platform.service.media_platform_request import MediaPlatformRequest


class ReplaceExtraMetadataBaseRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str, verb: str,
                 response_type: Type[Deserializable]):
        super().__init__(authenticated_http_client, verb, base_url + '/av/extra-metadata', response_type)
        self.source = None
        self.specification = None

    def set_specification(self,
                          specification: ReplaceAudioExtraMetadataSpecification) -> ReplaceExtraMetadataBaseRequest:
        self.specification = specification
        return self

    def set_source(self, source: Source) -> ReplaceExtraMetadataBaseRequest:
        self.source = source
        return self

    def _params(self) -> dict:
        return {
            'source': self.source.serialize(),
            'specification': self.specification.serialize()
        }


class ReplaceExtraMetadataSyncRequest(ReplaceExtraMetadataBaseRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, base_url, 'PUT', FileDescriptor)


class ReplaceExtraMetadataAsyncRequest(ReplaceExtraMetadataBaseRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, base_url, 'POST', ReplaceExtraMetadataJob)



class StreamErrorCode:
    success = 'success'
    invalid_stream_params = 'invalid_stream_params'
    stream_params_out_of_range = 'stream_params_out_of_range'
    dvr_upload_failed = 'dvr_upload_failed'
    stream_max_duration_reached = 'stream_max_duration_reached'



from __future__ import annotations

from media_platform.lang.serialization import Deserializable


class Color(Deserializable):
    def __init__(self, r: int, g: int, b: int, pixel_fraction: float, score: float):
        self.r = r
        self.g = g
        self.b = b
        self.pixel_fraction = pixel_fraction
        self.score = score

    @classmethod
    def deserialize(cls, data: dict) -> Color:
        return Color(data['r'], data['g'], data['b'], data['pixelFraction'], data['score'])



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.flow_control_service.abort_flow_request import AbortFlowRequest
from media_platform.service.flow_control_service.flow_invocation_request import FlowInvocationRequest
from media_platform.service.flow_control_service.flow_state_request import FlowStateRequest
from media_platform.service.media_platform_service import MediaPlatformService


class FlowControlService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient):
        super().__init__(domain, authenticated_http_client)

    def invoke_flow_request(self) -> FlowInvocationRequest:
        return FlowInvocationRequest(self._authenticated_http_client, self._base_url)

    def flow_state_request(self) -> FlowStateRequest:
        return FlowStateRequest(self._authenticated_http_client, self._base_url)

    def abort_flow_request(self) -> AbortFlowRequest:
        return AbortFlowRequest(self._authenticated_http_client, self._base_url)



import requests

from media_platform.exception.bad_gateway_exception import BadGatewayException
from media_platform.exception.conflict_exception import ConflictException
from media_platform.exception.server_error_exception import ServerErrorException
from media_platform.lang.serialization import Deserializable
from media_platform.exception.forbidden_exception import ForbiddenException
from media_platform.exception.media_platform_exception import MediaPlatformException
from media_platform.exception.not_found_exception import NotFoundException
from media_platform.exception.unauthorized_exception import UnauthorizedException
from media_platform.service.rest_result import RestResult


class ResponseProcessor:

    @staticmethod
    def process(response: requests.Response, payload_type: Deserializable = None) -> object or None:
        if 200 <= response.status_code <= 299:
            return ResponseProcessor._handle_success(response, payload_type)
        else:
            ResponseProcessor._handle_error(response)

    @classmethod
    def _handle_success(cls, response: requests.Response,
                        payload_type: Deserializable or [Deserializable]) -> object or None:
        try:
            rest_result = RestResult.deserialize(response.json())
            rest_result.raise_for_code()

            if rest_result.payload is None:
                return None

            if payload_type is None:
                raise MediaPlatformException('Unexpected payload (expected None)')

            payload = payload_type.deserialize(rest_result.payload)
            payload.seen_by = response.headers.get('X-Seen-By')
            return payload

        except (ValueError, KeyError) as e:
            raise MediaPlatformException('Bad response format', e)

    @classmethod
    def _handle_error(cls, response: requests.Response):
        try:
            rest_result = RestResult.deserialize(response.json())
            message = rest_result.message
        except (ValueError, KeyError):
            message = response.content

        if response.status_code == 401:
            raise UnauthorizedException(message)

        elif response.status_code == 403:
            raise ForbiddenException(message)

        elif response.status_code == 404:
            raise NotFoundException(message)

        elif response.status_code == 409:
            raise ConflictException(message)

        elif response.status_code == 500:
            raise ServerErrorException(message)

        elif response.status_code == 502:
            raise BadGatewayException(message)

        else:
            raise MediaPlatformException(message)



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.file_descriptor import FileDescriptor


class Source(Serializable, Deserializable):
    def __init__(self, path: str = None, file_id: str = None):
        self._validate(path, file_id)
        self.path = path
        self.file_id = file_id

    @classmethod
    def deserialize(cls, data: dict) -> Source:
        return Source(data.get('path'), data.get('fileId'))

    def serialize(self) -> dict:
        return {
            'path': self.path,
            'fileId': self.file_id
        }

    @staticmethod
    def _validate(path: str, file_id: str):
        if not (path or file_id):
            raise ValueError('path or file id must be specified')

        if path:
            FileDescriptor.path_validator(path)



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.live_service.enforced_stream_params import EnforcedStreamParams
from media_platform.service.live_service.geo_location import GeoLocation
from media_platform.service.live_service.live_stream import LiveStream
from media_platform.service.live_service.stream_dvr import StreamDVR
from media_platform.service.live_service.stream_protocol import StreamProtocol
from media_platform.service.live_service.stream_state_notification import StreamStateNotification
from media_platform.service.live_service.stream_type import StreamType
from media_platform.service.media_platform_request import MediaPlatformRequest


class OpenStreamRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/live/streams', LiveStream)
        self.protocol: StreamProtocol or None = None
        self.dvr: StreamDVR or None = None
        self.geo: GeoLocation or None = None
        self.max_stream_time_sec: int or None = None
        self.state_notification: StreamStateNotification or None = None
        self.stream_type: StreamType = StreamType.event
        self.connect_timeout: int or None = None
        self.reconnect_timeout: int or None = None
        self.enforced_stream_params: EnforcedStreamParams or None = None

    def set_protocol(self, protocol: StreamProtocol) -> OpenStreamRequest:
        self.protocol = protocol
        return self

    def set_dvr(self, dvr: StreamDVR) -> OpenStreamRequest:
        self.dvr = dvr
        return self

    def set_geo(self, geo: GeoLocation) -> OpenStreamRequest:
        self.geo = geo
        return self

    def set_max_stream_time_sec(self, max_stream_time_sec) -> OpenStreamRequest:
        self.max_stream_time_sec = max_stream_time_sec
        return self

    def set_state_notification(self, state_notification: StreamStateNotification) -> OpenStreamRequest:
        self.state_notification = state_notification
        return self

    def set_stream_type(self, stream_type: StreamType) -> OpenStreamRequest:
        self.stream_type = stream_type
        return self

    def set_connect_timeout(self, connect_timeout: int) -> OpenStreamRequest:
        self.connect_timeout = connect_timeout
        return self

    def set_reconnect_timeout(self, reconnect_timeout: int) -> OpenStreamRequest:
        self.reconnect_timeout = reconnect_timeout
        return self

    def set_enforced_stream_params(self, enforced_stream_params: EnforcedStreamParams) -> OpenStreamRequest:
        self.enforced_stream_params = enforced_stream_params
        return self

    def _params(self) -> dict:
        return {
            'protocol': self.protocol,
            'maxStreamingSec': self.max_stream_time_sec,
            'streamType': self.stream_type,
            'connectTimeout': self.connect_timeout,
            'reconnectTimeout': self.reconnect_timeout,
            'enforcedStreamParams': self.enforced_stream_params.serialize() if self.enforced_stream_params else None,
            'stateNotification': self.state_notification.serialize() if self.state_notification else None,
            'dvr': self.dvr.serialize() if self.dvr else None,
            'geo': self.geo.serialize() if self.geo else None
        }



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_service import MediaPlatformService
from media_platform.service.scanner_service.scan_file_request import ScanFileRequest


class ScannerService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient):
        super(ScannerService, self).__init__(domain, authenticated_http_client)

    def scan_file_request(self) -> ScanFileRequest:
        return ScanFileRequest(self._authenticated_http_client, self._base_url)



from __future__ import annotations

import json

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.callback import Callback
from media_platform.service.file_descriptor import ACL, FileDescriptor, FileMimeType
from media_platform.service.file_service.upload_configuration_request import UploadConfigurationRequest
from media_platform.service.lifecycle import Lifecycle
from media_platform.service.media_platform_request import MediaPlatformRequest


class UploadFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url, FileDescriptor)

        self.path = None
        self.mime_type = FileMimeType.defualt
        self.acl = ACL.public
        self.size = None
        self.lifecycle = None
        self.callback = None
        self.bucket = None
        self.response_processor = None
        self.filename = 'filename'
        self.content = None

    def set_path(self, path: str) -> UploadFileRequest:
        self.path = path
        return self

    def set_mime_type(self, mime_type: str) -> UploadFileRequest:
        self.mime_type = mime_type
        return self

    def set_acl(self, acl: ACL) -> UploadFileRequest:
        self.acl = acl
        return self

    def set_size(self, size: int) -> UploadFileRequest:
        self.size = size
        return self

    def set_lifecycle(self, lifecycle: Lifecycle) -> UploadFileRequest:
        self.lifecycle = lifecycle
        return self

    def set_callback(self, callback: Callback) -> UploadFileRequest:
        self.callback = callback
        return self

    def set_bucket(self, bucket: str) -> UploadFileRequest:
        self.bucket = bucket
        return self

    def override_response_processor(self, response_processor: callable) -> UploadFileRequest:
        self.response_processor = response_processor
        return self

    def set_filename(self, filename: str) -> UploadFileRequest:
        self.filename = filename
        return self

    def set_content(self, content: str) -> UploadFileRequest:
        self.content = content
        return self

    def validate(self):
        FileDescriptor.path_validator(self.path)
        FileDescriptor.acl_validator(self.acl)

    def execute(self) -> FileDescriptor:
        self.validate()

        config = UploadConfigurationRequest(self.authenticated_http_client, self.url).set_path(
            self.path
        ).set_acl(self.acl).set_mime_type(self.mime_type).set_callback(self.callback).set_size(
            self.size
        ).set_bucket(self.bucket).execute()

        params = self._params()
        return self.authenticated_http_client.post_data(config.upload_url, self.content, self.mime_type, params,
                                                        FileDescriptor, self.filename, self.response_processor)

    def _params(self) -> dict:
        return {
            'lifecycle': json.dumps(self.lifecycle.serialize()) if self.lifecycle else None,
        }



from __future__ import annotations

from media_platform.lang.serialization import Deserializable
from media_platform.metadata.audio.attached_image import AttachedImage
from media_platform.metadata.audio.lyrics import Lyrics


class AudioExtra(Deserializable):
    def __init__(self, track_name: str = None, artist: str = None, album_name: str = None, track_number: str = None,
                 genre: str = None, composer: str = None, year: str = None, images: [AttachedImage] = None,
                 lyrics: Lyrics = None):
        self.track_name = track_name
        self.artist = artist
        self.album_name = album_name
        self.track_number = str(track_number) if track_number is not None else None
        self.genre = genre
        self.composer = composer
        self.year = year

        self.images = images or []
        self.lyrics = lyrics

    @classmethod
    def deserialize(cls, data: dict) -> AudioExtra:
        lyrics_data = data.get('lyrics')

        images_data = data.get('images')
        if images_data:
            images = [AttachedImage.deserialize(image_data) for image_data in images_data if image_data]
        else:
            images = []

        lyrics = Lyrics.deserialize(lyrics_data) if lyrics_data else None

        return cls(data.get('trackName'), data.get('artist'), data.get('albumName'), data.get('trackNumber'),
                   data.get('genre'), data.get('composer'), data.get('year'), images, lyrics)



from media_platform.lang.serialization import Deserializable, Serializable


class PackageName:
    hls = 'hls'
    dash = 'dash'


class StreamPlayback(Serializable, Deserializable):
    def __init__(self, package_name: PackageName, path: str):
        self.package_name = package_name
        self.path = path

    @classmethod
    def deserialize(cls, data: dict):
        return cls(data['packageName'], data['path'])

    def serialize(self) -> dict:
        return {
            'packageName': self.package_name,
            'path': self.path
        }



from __future__ import annotations

from media_platform.job.job import Job
from media_platform.job.specification import Specification
from media_platform.service.audio_service.audio_extra_metadata import AudioExtraMetadata
from media_platform.service.destination import Destination


class ReplaceAudioExtraMetadataSpecification(Specification):
    def __init__(self, destination: Destination, audio_extra_metadata: AudioExtraMetadata):
        self.destination = destination
        self.audio_extra_metadata = audio_extra_metadata

    @classmethod
    def deserialize(cls, data: dict) -> ReplaceAudioExtraMetadataSpecification:
        return cls(
            Destination.deserialize(data['destination']),
            AudioExtraMetadata.deserialize(data['audioExtraMetadata'])
        )

    def serialize(self) -> dict:
        return {
            'destination': self.destination.serialize(),
            'audioExtraMetadata': self.audio_extra_metadata.serialize()
        }


class ReplaceExtraMetadataJob(Job):
    type = 'urn:job:av.replace_extra_metadata'
    specification_type = ReplaceAudioExtraMetadataSpecification



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable


class Lyrics(Serializable, Deserializable):
    def __init__(self, text: str, language: str = None, description: str = None):
        self.description = description
        self.language = language
        self.text = text

    def serialize(self) -> dict:
        return {
            'text': self.text,
            'description': self.description,
            'language': self.language,
        }

    @classmethod
    def deserialize(cls, data: dict) -> Lyrics:
        return cls(data['text'], data.get('language') or data.get('lang'), data.get('description'))



from __future__ import annotations

from media_platform.job.specification import Specification


class VideoQuality:
    res_144p = '144p'
    res_240p = '240p'
    res_360p = '360p'
    res_480p = '480p'
    res_720p = '720p'
    res_1080p = '1080p'
    res_1440p = '1440p'
    res_2160p = '2160p'

    # order matters!
    values = [res_144p, res_240p, res_360p, res_480p, res_720p, res_1080p, res_1440p, res_2160p]

    @classmethod
    def has_value(cls, value: str or VideoQuality) -> bool:
        return value in cls.values


class VideoQualityRange(Specification):
    def __init__(self, minimum: VideoQuality, maximum: VideoQuality):
        self.minimum = minimum
        self.maximum = maximum

    @classmethod
    def deserialize(cls, data: dict) -> VideoQualityRange:
        return VideoQualityRange(data['minimum'], data['maximum'])

    def serialize(self) -> dict:
        return {
            'minimum': self.minimum,
            'maximum': self.maximum
        }

    def validate(self):
        if not (self.maximum and self.minimum):
            raise ValueError('must define minimum and maximum')

        if not VideoQuality.has_value(str(self.minimum)):
            raise ValueError('minimum value %s not supported' % self.minimum)

        if not VideoQuality.has_value(str(self.maximum)):
            raise ValueError('maximum value %s not supported' % self.maximum)

        if VideoQuality.values.index(str(self.maximum)) < VideoQuality.values.index(str(self.minimum)):
            raise ValueError('maximum %s is greater than minimum %s' % (self.maximum, self.minimum))



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.metadata.audio.lyrics import Lyrics


class Image(Serializable, Deserializable):
    def __init__(self, url: str, mime_type: str = None, description: str = None):
        self.url = url
        self.mime_type = mime_type
        self.description = description

    def serialize(self) -> dict:
        return {
            'url': self.url,
            'mimeType': self.mime_type,
            'description': self.description
        }

    @classmethod
    def deserialize(cls, data: dict) -> Image:
        return cls(data['url'], data.get('mimeType'), data.get('description'))


class AudioExtraMetadata(Serializable, Deserializable):
    def __init__(self, track_name: str = None, artist: str = None, album_name: str = None, track_number: str = None,
                 genre: str = None, composer: str = None, year: str = None, image: Image = None, lyrics: Lyrics = None):
        self.track_name = track_name
        self.artist = artist
        self.album_name = album_name
        self.track_number = track_number
        self.genre = genre
        self.composer = composer
        self.year = year
        self.image = image
        self.lyrics = lyrics

    def serialize(self) -> dict:
        data = {
            'trackName': self.track_name,
            'artist': self.artist,
            'albumName': self.album_name,
            'trackNumber': self.track_number,
            'genre': self.genre,
            'composer': self.composer,
            'year': self.year,
            'image': self.image.serialize() if self.image else None,
            'lyrics': self.lyrics.serialize() if self.lyrics else None,
        }

        return self._omit_empty(data)

    @classmethod
    def deserialize(cls, data: dict) -> AudioExtraMetadata:
        image_data = data.get('image')
        lyrics_data = data.get('lyrics')
        image = Image.deserialize(image_data) if image_data else None
        lyrics = Lyrics.deserialize(lyrics_data) if lyrics_data else None
        return cls(data.get('trackName'), data.get('artist'), data.get('albumName'), data.get('trackNumber'),
                   data.get('genre'), data.get('composer'), data.get('year'), image, lyrics)

    @staticmethod
    def _omit_empty(data: dict) -> dict:
        return {k: v for k, v in data.items() if v}



from __future__ import annotations

from media_platform.metadata.file_metadata import FileMetadata, MediaType
from media_platform.metadata.video.video_basic import VideoBasic
from media_platform.service.file_descriptor import FileDescriptor


class VideoFileMetadata(FileMetadata):
    def __init__(self, file_descriptor: FileDescriptor, basic: VideoBasic = None):
        super().__init__(MediaType.video, file_descriptor, basic)

    @classmethod
    def deserialize(cls, data: dict) -> VideoFileMetadata:
        if data['mediaType'] != MediaType.video:
            raise ValueError('not video metadata')

        basic_data = data.get('basic')

        file_descriptor = FileDescriptor.deserialize(data['fileDescriptor'])
        basic = VideoBasic.deserialize(basic_data) if basic_data else None

        return VideoFileMetadata(file_descriptor, basic)



from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_service.copy_file_request import CopyFileRequest
from media_platform.service.file_service.create_file_request import CreateFileRequest
from media_platform.service.file_service.create_files_request import CreateFilesRequest
from media_platform.service.file_service.delete_file_request import DeleteFileRequest
from media_platform.service.file_service.download_file_request import DownloadFileRequest
from media_platform.service.file_service.extract_metadata_request import ExtractMetadataRequest
from media_platform.service.file_service.file_list_request import FileListRequest
from media_platform.service.file_service.file_metadata_request import FileMetadataRequest
from media_platform.service.file_service.file_request import FileRequest
from media_platform.service.file_service.import_file_request import ImportFileRequest
from media_platform.service.file_service.update_file_request import UpdateFileRequest
from media_platform.service.file_service.upload_configuration_request import UploadConfigurationRequest
from media_platform.service.file_service.upload_file_request import UploadFileRequest
from media_platform.service.media_platform_service import MediaPlatformService


class FileService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient, app_id: str,
                 app_authenticator: AppAuthenticator):
        super().__init__(domain, authenticated_http_client)
        self._app_id = app_id
        self._app_authenticator = app_authenticator

    def file_request(self) -> FileRequest:
        return FileRequest(self._authenticated_http_client, self._base_url)

    def create_file_request(self) -> CreateFileRequest:
        return CreateFileRequest(self._authenticated_http_client, self._base_url)

    def update_file_request(self) -> UpdateFileRequest:
        return UpdateFileRequest(self._authenticated_http_client, self._base_url)

    def create_files_request(self) -> CreateFilesRequest:
        return CreateFilesRequest(self._authenticated_http_client, self._base_url)

    def upload_configuration_request(self) -> UploadConfigurationRequest:
        return UploadConfigurationRequest(self._authenticated_http_client, self._base_url)

    def upload_file_request(self) -> UploadFileRequest:
        return UploadFileRequest(self._authenticated_http_client, self._base_url)

    def import_file_request(self) -> ImportFileRequest:
        return ImportFileRequest(self._authenticated_http_client, self._base_url)

    def delete_file_request(self) -> DeleteFileRequest:
        return DeleteFileRequest(self._authenticated_http_client, self._base_url)

    def copy_file_request(self) -> CopyFileRequest:
        return CopyFileRequest(self._authenticated_http_client, self._base_url)

    def download_file_request(self) -> DownloadFileRequest:
        return DownloadFileRequest(self._app_id, self._app_authenticator, self._base_url)

    def file_list_request(self) -> FileListRequest:
        return FileListRequest(self._authenticated_http_client, self._base_url)

    def file_metadata_request(self) -> FileMetadataRequest:
        return FileMetadataRequest(self._authenticated_http_client, self._base_url)

    def extract_metadata_request(self) -> ExtractMetadataRequest:
        return ExtractMetadataRequest(self._authenticated_http_client, self._base_url)



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_service import MediaPlatformService
from media_platform.service.transcode_service.playlist_request import PlaylistRequest
from media_platform.service.transcode_service.transcode_request import TranscodeRequest


class TranscodeService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient):
        super(TranscodeService, self).__init__(domain, authenticated_http_client)

    def transcode_request(self) -> TranscodeRequest:
        return TranscodeRequest(self._authenticated_http_client, self._base_url)

    def playlist_request(self) -> PlaylistRequest:
        return PlaylistRequest(self._domain)

    # todo: packager ?



from __future__ import annotations

from media_platform.lang.serialization import Serializable


class Inline(Serializable):
    def __init__(self, file_name: str = None):
        self.file_name = file_name

    def set_file_name(self, file_name: str) -> Inline:
        self.file_name = file_name
        return self

    def serialize(self) -> dict:
        return {
            'filename': self.file_name
        }



from __future__ import annotations

from media_platform.error_code import ErrorCode
from media_platform.job.job_type import JobType
from media_platform.job.result.convert_font_result import ConvertFontResult
from media_platform.job.result.create_archive_result import CreateArchiveResult
from media_platform.job.result.extract_archive_result import ExtractArchiveResult
from media_platform.job.result.extract_poster_result import ExtractPosterResult
from media_platform.job.result.extract_storyboard_result import ExtractStoryboardResult
from media_platform.job.result.import_file_result import ImportFileResult
from media_platform.job.result.job_callback_failed_result import JobCallbackFailedResult
from media_platform.job.result.job_result import JobResult
from media_platform.job.result.job_timeout_result import JobTimeoutResult
from media_platform.job.result.replace_audio_extra_metadata_result import ReplaceAudioExtraMetadataResult
from media_platform.job.result.subset_font_result import SubsetFontResult
from media_platform.job.result.transcode_result import TranscodeResult


class JobResultDeserializer:
    _result_classes = [
        CreateArchiveResult,
        ExtractArchiveResult,
        ExtractPosterResult,
        ExtractStoryboardResult,
        ImportFileResult,
        ReplaceAudioExtraMetadataResult,
        TranscodeResult,
        ConvertFontResult,
        SubsetFontResult,
    ]

    _type_to_class = {c.type: c for c in _result_classes}

    @classmethod
    def deserialize(cls, job_type: JobType, data: dict) -> JobResult or None:
        if data['code'] == ErrorCode.job_callback_failed:
            job_result_class = JobCallbackFailedResult

        elif data['code'] == ErrorCode.job_timeout:
            job_result_class = JobTimeoutResult

        else:
            job_result_class = cls._type_to_class.get(job_type, JobResult)

        return job_result_class.deserialize(data)



from __future__ import annotations

from media_platform.service.source import Source
from media_platform.job.specification import Specification


class AddSourcesSpecification(Specification):
    def __init__(self, sources: [Source]):
        self.sources = sources

    def serialize(self) -> dict:
        return {
            'sources': [s.serialize() for s in self.sources]
        }

    @classmethod
    def deserialize(cls, data: dict) -> AddSourcesSpecification:
        return cls([Source.deserialize(s) for s in data['sources']])



from abc import ABC

from typing import Type

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.lang.serialization import Deserializable


class MediaPlatformRequest(ABC):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, method: str, url: str,
                 response_payload_type: Type[Deserializable] = None):
        self.authenticated_http_client = authenticated_http_client
        self.method = method
        self.url = url
        self.response_payload_type = response_payload_type

    def execute(self) -> Deserializable or None:
        self.validate()

        if self.method == 'GET':
            return self.authenticated_http_client.get(self.url, self._params(), self.response_payload_type)
        elif self.method == 'POST':
            return self.authenticated_http_client.post(self.url, self._params(), self.response_payload_type)
        elif self.method == 'PUT':
            return self.authenticated_http_client.put(self.url, self._params(), self.response_payload_type)
        elif self.method == 'DELETE':
            return self.authenticated_http_client.delete(self.url, self._params(), self.response_payload_type)
        else:
            raise ValueError('method not supported %s' % self.method)

    # override for request pre-flight check
    def validate(self):
        pass

    # noinspection PyMethodMayBeStatic
    def _params(self) -> dict:
        return {}



class MediaPlatformException(Exception):
    def __init__(self, message: str = None, cause: Exception = None):
        if cause:
            message = '%s: %s: %s' % (message, str(type(cause).__name__), str(cause))

        super(MediaPlatformException, self).__init__(message)
        self.cause = cause



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.job_service.job_group_request import JobGroupRequest
from media_platform.service.job_service.job_list_request import JobListRequest
from media_platform.service.job_service.job_request import JobRequest
from media_platform.service.media_platform_service import MediaPlatformService


class JobService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient):
        super().__init__(domain, authenticated_http_client)

    def job_request(self) -> JobRequest:
        return JobRequest(self._authenticated_http_client, self._base_url)

    def job_group_request(self) -> JobGroupRequest:
        return JobGroupRequest(self._authenticated_http_client, self._base_url)

    def job_list_request(self) -> JobListRequest:
        return JobListRequest(self._authenticated_http_client, self._base_url)



from jose import jwt

from media_platform.auth.token import Token


def sign_jwt(token: Token, secret: str) -> str:
    return jwt.encode(token.to_claims(), secret, algorithm='HS256')


def decode_jwt(shared_secret: str, urn: str, signed_token: str) -> Token:
    claims = jwt.decode(signed_token, shared_secret, subject=urn, options={
        'verify_aud': False
    })

    return Token.from_claims(claims)



from __future__ import annotations

from media_platform.job.job import Job
from media_platform.job.specification import Specification
from media_platform.job.transcode.audio_qualities import AudioQuality
from media_platform.job.transcode.clipping import Clipping
from media_platform.job.transcode.stream_specification import StreamSpecification
from media_platform.job.transcode.video_qualities import VideoQualityRange, VideoQuality
from media_platform.service.destination import Destination


class TranscodeSpecification(Specification):
    def __init__(self, destination: Destination, video: StreamSpecification = None, audio: StreamSpecification = None,
                 quality_range: VideoQualityRange = None, quality: AudioQuality or VideoQuality = None,
                 clipping: Clipping = None):
        self.destination = destination
        self.video = video
        self.audio = audio
        self.quality_range = quality_range
        self.quality = quality
        self.clipping = clipping

    @classmethod
    def deserialize(cls, data: dict) -> TranscodeSpecification:
        destination = Destination.deserialize(data['destination'])

        video_data = data.get('video')
        video = StreamSpecification.deserialize(video_data) if video_data else None

        audio_data = data.get('audio')
        audio = StreamSpecification.deserialize(audio_data) if audio_data else None

        quality_range_data = data.get('qualityRange')
        quality_range = VideoQualityRange.deserialize(quality_range_data) if quality_range_data else None

        quality = data.get('quality')

        clipping_data = data.get('clipping')
        clipping = Clipping.deserialize(clipping_data) if clipping_data else None

        return TranscodeSpecification(destination, video, audio, quality_range, quality, clipping)

    def serialize(self) -> dict:
        return {
            'destination': self.destination.serialize(),
            'video': self.video.serialize() if self.video else None,
            'audio': self.audio.serialize() if self.audio else None,
            'qualityRange': self.quality_range.serialize() if self.quality_range else None,
            'quality': self.quality,
            'clipping': self.clipping.serialize() if self.clipping else None
        }

    def validate(self):
        stream_specified = (self.video or self.audio)
        quality_specified = (self.quality_range or self.quality)

        if self.quality_range:
            self.quality_range.validate()

        if self.video:
            self.video.validate()

        if self.audio:
            self.video.validate()

        if stream_specified and quality_specified:
            raise ValueError('Either stream specification or quality may be specified, not both')

        if self.quality_range and self.quality:
            raise ValueError('Either quality range or quality may be specified, not both')

        if self.quality and not VideoQuality.has_value(self.quality) and not AudioQuality.has_value(self.quality):
            raise ValueError('Quality %s is not supported' % self.quality)

        if not stream_specified and not quality_specified and not self.clipping:
            raise ValueError('Either video, audio, quality range, quality or clipping must be specified')


class TranscodeJob(Job):
    type = 'urn:job:av.transcode'
    specification_type = TranscodeSpecification



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_descriptor import FileType
from media_platform.service.file_service.file_list import FileList
from media_platform.service.list_request import _ListRequest, OrderBy, OrderDirection


class FileListRequest(_ListRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, base_url + '/files/ls_dir', FileList)
        self.path = None
        self.recursive = False
        self.type = FileType.file

    def set_path(self, path: str) -> FileListRequest:
        self.path = path
        return self

    def set_recursive(self, recursive: bool) -> FileListRequest:
        self.recursive = recursive
        return self

    def set_type(self, file_type: FileType) -> FileListRequest:
        self.type = file_type
        return self

    def set_next_page_token(self, next_page_token: str) -> FileListRequest:
        return super().set_next_page_token(next_page_token)

    def set_page_size(self, page_size: int) -> FileListRequest:
        return super().set_page_size(page_size)

    def set_order_by(self, order_by: OrderBy) -> FileListRequest:
        return super().set_order_by(order_by)

    def set_order_direction(self, order_direction: OrderDirection) -> FileListRequest:
        return super().set_order_direction(order_direction)

    def execute(self) -> FileList:
        return super().execute()

    def _params(self) -> dict:
        params = super()._params()

        params['path'] = self.path

        if self.recursive:
            params['r'] = self.recursive

        if self.type:
            params['type'] = self.type

        return params



from __future__ import annotations

from media_platform.job.extract_archive.extraction_report import ExtractionReport
from media_platform.job.job import Job
from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.service.source import Source


class ExtractArchiveSpecification(Specification):
    def __init__(self, source: Source, destination: Destination, extraction_report: ExtractionReport = None):
        self.source = source
        self.destination = destination
        self.extraction_report = extraction_report

    @classmethod
    def deserialize(cls, data: dict) -> ExtractArchiveSpecification:
        extraction_report_data = data.get('extractedFilesReport')
        if extraction_report_data:
            extraction_report = ExtractionReport.deserialize(extraction_report_data)
        else:
            extraction_report = None

        source_data = data.get('source')
        source = Source.deserialize(source_data) if source_data else None
        return ExtractArchiveSpecification(source,
                                           Destination.deserialize(data['destination']),
                                           extraction_report)

    def serialize(self) -> dict:
        return {
            'source': self.source.serialize() if self.source else None,
            'destination': self.destination.serialize(),
            'extractedFilesReport': self.extraction_report.serialize() if self.extraction_report else None
        }


class ExtractArchiveJob(Job):
    type = JobType.extract_archive
    specification_type = ExtractArchiveSpecification



from __future__ import annotations

from media_platform.metadata.features.rectangle import Rectangle


class CropHint(Rectangle):
    def __init__(self, x: int, y: int, width: int, height: int, confidence: float, importance_fraction: float):
        super().__init__(x, y, width, height)
        self.confidence = confidence
        self.importance_fraction = importance_fraction

    @classmethod
    def deserialize(cls, data: dict) -> CropHint:
        return CropHint(data['x'], data['y'], data['width'], data['height'], data['confidence'],
                        data['importanceFraction'])



from __future__ import annotations

from media_platform.job.specification import Specification
from media_platform.lang.serialization import Serializable, Deserializable


class AudioSpecification(Specification):
    def __init__(self, codec: AudioCodec, channels: Channels = None, sampling: AudioSampling = None):
        self.codec = codec
        self.channels = channels
        self.sampling = sampling

    @classmethod
    def deserialize(cls, data: dict) -> AudioSpecification:
        codec = AudioCodec.deserialize(data['codec'])

        sampling_info = data.get('sampling')
        sampling = AudioSampling.deserialize(sampling_info) if sampling_info else None

        return AudioSpecification(codec, data.get('channels'), sampling)

    def serialize(self):
        return {
            'codec': self.codec.serialize(),
            'channels': self.channels,
            'sampling': self.sampling.serialize() if self.sampling else None
        }

    def validate(self):
        if self.channels and not Channels.has_value(self.channels):
            raise ValueError('Channels must be one of %s' % ', '.join(Channels.values))


class Channels:
    stereo = 'stereo'
    mono = 'mono'
    joint = 'joint'

    values = [stereo, mono, joint]

    @classmethod
    def has_value(cls, value: str or Channels) -> bool:
        return value in cls.values


class AudioCodec(Serializable, Deserializable):
    def __init__(self, name: str, cbr: float = None, abr: float = None, vbr: float = None,
                 profile: AudioProfile = None):
        self.name = name
        self.cbr = cbr
        self.abr = abr
        self.vbr = vbr
        self.profile = profile

    @classmethod
    def deserialize(cls, data: dict) -> AudioCodec:
        profile_info = data.get('profile')
        profile = AudioProfile.deserialize(profile_info) if profile_info else None

        return AudioCodec(data['name'], data.get('cbr'), data.get('abr'), data.get('vbr'), profile)

    def serialize(self) -> dict:
        return {
            'name': self.name,
            'cbr': self.cbr,
            'abr': self.abr,
            'vbr': self.vbr,
            'profile': self.profile.serialize() if self.profile else None
        }


class AudioSampling(Serializable, Deserializable):
    def __init__(self, rate: int, size: int):
        self.rate = rate
        self.size = size

    @classmethod
    def deserialize(cls, data: dict) -> AudioSampling:
        return AudioSampling(data.get('rate'), data.get('size'))

    def serialize(self) -> dict:
        return {
            'rate': self.rate,
            'size': self.size
        }


class AudioProfile(Serializable, Deserializable):
    def __init__(self, name: str, rate: float):
        self.name = name
        self.rate = rate

    @classmethod
    def deserialize(cls, data: dict) -> AudioProfile:
        return AudioProfile(data['name'], data['rate'])

    def serialize(self) -> dict:
        return {
            'name': self.name,
            'rate': self.rate
        }



from __future__ import annotations

from media_platform.lang.serialization import Deserializable, Serializable
from media_platform.service.flow_control_service.component import ComponentType


class FlowError(Deserializable, Serializable):
    def __init__(self, component_type: ComponentType, component_key: str, message: str = None):
        self.component_type = component_type
        self.component_key = component_key
        self.message = message

    @classmethod
    def deserialize(cls, data: dict) -> FlowError:
        return FlowError(data['componentType'], data['componentKey'], data.get('message'))

    def serialize(self) -> dict:
        return {
            'componentType': self.component_type,
            'componentKey': self.component_key,
            'message': self.message
        }



from __future__ import annotations

from media_platform.job.job import Job
from media_platform.lang.serialization import Deserializable
# noinspection PyProtectedMember
from media_platform.job.job_deserializer import _JobDeserializer


class JobList(Deserializable):
    def __init__(self, next_page_token: str, jobs: [Job]):
        self.next_page_token = next_page_token
        self.jobs = jobs

    @classmethod
    def deserialize(cls, data: dict) -> JobList:
        jobs = [_JobDeserializer.deserialize(j) for j in data['jobs']]

        return JobList(data['nextPageToken'], jobs)



from __future__ import annotations


class ContentDisposition:
    class Type:
        attachment = 'attachment'
        inline = 'inline'

    def __init__(self, file_name: str = None, disposition_type: Type = Type.attachment):
        self.file_name = file_name
        self.disposition_type = disposition_type

    def set_file_name(self, file_name: str) -> ContentDisposition:
        self.file_name = file_name
        return self

    def set_type(self, disposition_type: Type) -> ContentDisposition:
        self.disposition_type = disposition_type
        return self

    def serialize(self) -> dict:
        return {
            'filename': self.file_name,
            'type': self.disposition_type
        }



from __future__ import annotations

from media_platform.lang.serialization import Deserializable, Serializable


class VideoStream(Serializable, Deserializable):
    def __init__(self, index: int, width: int, height: int, avg_frame_rate: str, r_frame_rate: str,
                 display_aspect_ratio: str, sample_aspect_ratio: str,
                 rotate: int, duration: int, bitrate: int, codec_tag: str, codec_name: str, codec_long_name: str,
                 pixel_format: str, field_order: str = None, disposition: [str] = None):
        self.index = int(index)
        self.width = int(width) if width else None
        self.height = int(height) if height else None
        self.avg_frame_rate = avg_frame_rate
        self.r_frame_rate = r_frame_rate
        self.display_aspect_ratio = display_aspect_ratio
        self.sample_aspect_ratio = sample_aspect_ratio
        self.rotate = int(rotate) if rotate is not None else None
        self.duration = duration
        self.bitrate = int(bitrate) if bitrate else None
        self.codec_tag = codec_tag
        self.codec_name = codec_name
        self.codec_long_name = codec_long_name
        self.pixel_format = pixel_format
        self.field_order = field_order
        self.disposition = disposition or []

    @classmethod
    def deserialize(cls, data: dict) -> VideoStream:
        return VideoStream(data.get('index'),
                           data.get('width'),
                           data.get('height'),
                           data.get('avgFrameRate'),
                           data.get('rFrameRate'),
                           data.get('displayAspectRatio'),
                           data.get('sampleAspectRatio', '1:1'),
                           data.get('rotate'),
                           data.get('duration'),
                           data.get('bitrate'),
                           data.get('codecTag'),
                           data.get('codecName'),
                           data.get('codecLongName'),
                           data.get('pixelFormat'),
                           data.get('fieldOrder'),
                           data.get('disposition')
                           )

    def serialize(self) -> dict:
        return {
            'index': self.index,
            'width': self.width,
            'height': self.height,
            'avgFrameRate': self.avg_frame_rate,
            'rFrameRate': self.r_frame_rate,
            'displayAspectRatio': self.display_aspect_ratio,
            'sampleAspectRatio': self.sample_aspect_ratio,
            'rotate': self.rotate,
            'duration': self.duration,
            'bitrate': self.bitrate,
            'codecTag': self.codec_tag,
            'codecName': self.codec_name,
            'codecLongName': self.codec_long_name,
            'pixelFormat': self.pixel_format,
            'fieldOrder': self.field_order,
            'disposition': self.disposition
        }



from __future__ import annotations

from media_platform.service.file_descriptor import FileDescriptor
from media_platform.job.result.job_result import JobResult


class ConvertFontResult(JobResult):
    def __init__(self, code: int = None, message: str = None, file_descriptor: FileDescriptor = None):
        super().__init__(code, message)
        self.file_descriptor = file_descriptor

    @classmethod
    def deserialize(cls, data: dict or None) -> ConvertFontResult or None:
        if data is None:
            return None

        payload_data = data.get('payload')
        result = JobResult.deserialize(data)
        result.__class__ = ConvertFontResult
        result.file_descriptor = FileDescriptor.deserialize(payload_data) if payload_data else None
        return result

    def serialize(self) -> dict:
        data = super().serialize()
        data['payload'] = self.file_descriptor.serialize() if self.file_descriptor else None

        return data



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.job import Job
from media_platform.service.job_service.job_group_response import _JobGroupResponse
from media_platform.service.media_platform_request import MediaPlatformRequest


class JobGroupRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'GET', base_url + '/jobs/', _JobGroupResponse)
        self.group_id = None
        self._url = base_url + '/jobs/groups/'

    def set_group_id(self, group_id: str) -> JobGroupRequest:
        self.group_id = group_id
        return self

    def execute(self) -> [Job]:
        self.url = self._url + self.group_id

        return super().execute().jobs



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.job import Job
from media_platform.service.media_platform_request import MediaPlatformRequest

# noinspection PyProtectedMember
from media_platform.job.job_deserializer import _JobDeserializer


class JobRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'GET', base_url + '/jobs/', _JobDeserializer)
        self.job_id = None

        self._url = base_url + '/jobs/'

    def set_id(self, job_id: str) -> JobRequest:
        self.job_id = job_id
        return self

    def execute(self) -> Job:
        self.url = self._url + self.job_id

        return super().execute()



from __future__ import annotations

from datetime import timedelta

from media_platform.error_code import ErrorCode
from media_platform.job.result.job_result import JobResult


class JobTimeoutResult(JobResult):
    type = None

    def __init__(self, job_id: str, timeout: timedelta):
        super().__init__(ErrorCode.job_timeout, 'Job %s timed out after %s' % (job_id, timeout))



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable


class FontBasic(Serializable, Deserializable):
    def __init__(self, font_type: str = None, name: str = None, family: str = None):
        self.font_type = font_type
        self.name = name
        self.family = family

    @classmethod
    def deserialize(cls, data: dict) -> FontBasic:
        return FontBasic(data['fontType'], data.get('name'), data.get('family'))

    def serialize(self) -> dict:
        return {
            'fontType': self.font_type,
            'name': self.name,
            'family': self.family
        }



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.subset_font_job import SubsetFontSpecification
from media_platform.job.subset_font_job_group import SubsetFontJobGroup
from media_platform.service.callback import Callback
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class SubsetFontRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/fonts/subset', SubsetFontJobGroup)

        self.source = None
        self.specification = None
        self.callback = None

    def set_source(self, source: Source):
        self.source = source
        return self

    def set_specification(self, specification: SubsetFontSpecification):
        self.specification = specification
        return self

    def set_callback(self, callback: Callback):
        self.callback = callback
        return self

    def validate(self):
        self.specification.validate()

    def execute(self) -> SubsetFontJobGroup:
        return super().execute()

    def _params(self) -> dict:
        return {
            'source': self.source.serialize(),
            'specification': self.specification.serialize(),
            'jobCallback': self.callback.serialize() if self.callback else None
        }



from media_platform.auth.jwt_coder import sign_jwt, decode_jwt
from media_platform.auth.token import Token


class AppAuthenticator:
    def __init__(self, app_id: str, shared_secret: str):
        self._shared_secret = shared_secret
        self._app_urn = 'urn:app:' + app_id

    def default_token(self) -> Token:
        return Token(self._app_urn, self._app_urn)

    def default_signed_token(self) -> str:
        token = self.default_token()

        return self.sign_token(token)

    def sign_token(self, token: Token) -> str:
        return sign_jwt(token, self._shared_secret)

    def decode_token(self, signed_token: str) -> Token:
        return decode_jwt(self._shared_secret, self._app_urn, signed_token)



from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.archive_service.archive_service import ArchiveService
from media_platform.service.audio_service.audio_service import AudioService
from media_platform.service.file_service.file_service import FileService
from media_platform.service.flow_control_service.flow_control_service import FlowControlService
from media_platform.service.image_service.image_service import ImageService
from media_platform.service.job_service.job_service import JobService
from media_platform.service.live_service.live_service import LiveService
from media_platform.service.scanner_service.scanner_service import ScannerService
from media_platform.service.text_service.text_service import TextService
from media_platform.service.transcode_service.transcode_service import TranscodeService
from media_platform.service.video_service.video_service import VideoService


class MediaPlatformClient:
    def __init__(self, domain: str, app_id: str, shared_secret: str):
        self.app_authenticator = AppAuthenticator(app_id, shared_secret)
        authenticated_http_client = AuthenticatedHTTPClient(self.app_authenticator)

        self.file_service = FileService(domain, authenticated_http_client, app_id, self.app_authenticator)
        self.job_service = JobService(domain, authenticated_http_client)
        self.video_service = VideoService(domain, authenticated_http_client)
        self.archive_service = ArchiveService(domain, authenticated_http_client, app_id, self.app_authenticator)
        self.transcode_service = TranscodeService(domain, authenticated_http_client)
        self.flow_control_service = FlowControlService(domain, authenticated_http_client)
        self.image_service = ImageService(domain, authenticated_http_client, self.app_authenticator)
        self.audio_service = AudioService(domain, authenticated_http_client)
        self.text_service = TextService(domain, authenticated_http_client)
        self.live_service = LiveService(domain, authenticated_http_client)
        self.scanner_service = ScannerService(domain, authenticated_http_client)



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_descriptor import FileDescriptor, FileMimeType, FileType, ACL
from media_platform.service.media_platform_request import MediaPlatformRequest


class CreateFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/files', FileDescriptor)
        self.path: str or None = None
        self.mime_type: FileMimeType = FileMimeType.directory
        self.type: FileType = FileType.directory
        self.acl: ACL = ACL.public
        self.size: int = 0
        self.file_id: str or None = None
        self.bucket: str or None = None

    def set_path(self, path: str) -> CreateFileRequest:
        self.path = path
        return self

    def set_mime_type(self, mime_type: FileMimeType) -> CreateFileRequest:
        self.mime_type = mime_type
        return self

    def set_type(self, file_type: FileType) -> CreateFileRequest:
        self.type = file_type
        return self

    def set_acl(self, acl: ACL) -> CreateFileRequest:
        self.acl = acl
        return self

    def set_size(self, size: int) -> CreateFileRequest:
        self.size = size
        return self

    def set_id(self, file_id: str) -> CreateFileRequest:
        self.file_id = file_id
        return self

    def set_bucket(self, bucket: str) -> CreateFileRequest:
        self.bucket = bucket
        return self

    def _params(self) -> dict:
        return {
            'path': self.path,
            'mimeType': self.mime_type,
            'type': self.type,
            'acl': self.acl,
            'size': self.size,
            'id': self.file_id,
            'bucket': self.bucket
        }



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.import_file_job import ImportFileJob
from media_platform.service.callback import Callback
from media_platform.service.destination import Destination
from media_platform.service.file_service.external_authorization import ExternalAuthorization
from media_platform.service.media_platform_request import MediaPlatformRequest


class ImportFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/import/file', ImportFileJob)
        self.source_url = None
        self.external_authorization = None
        self.destination = None
        self.job_callback = None

    def set_source_url(self, source_url: str) -> ImportFileRequest:
        self.source_url = source_url
        return self

    def set_external_authorization(self, external_authorization: ExternalAuthorization) -> ImportFileRequest:
        self.external_authorization = external_authorization
        return self

    def set_destination(self, destination: Destination) -> ImportFileRequest:
        self.destination = destination
        return self

    def set_job_callback(self, job_callback: Callback) -> ImportFileRequest:
        self.job_callback = job_callback
        return self

    def execute(self) -> ImportFileJob:
        return super().execute()

    def _params(self) -> dict:
        return {
            'sourceUrl': self.source_url,
            'externalAuthorization': self.external_authorization.serialize() if self.external_authorization else None,
            'destination': self.destination.serialize(),
            'jobCallback': self.job_callback.serialize() if self.job_callback else None
        }



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.live_service.live_stream import LiveStream
from media_platform.service.media_platform_request import MediaPlatformRequest


class GetStreamRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'GET', base_url + '/live/stream/', LiveStream)
        self.id = None
        self._url = base_url + '/live/stream/'

    def set_id(self, stream_id: str) -> GetStreamRequest:
        self.id = stream_id
        return self

    def execute(self) -> LiveStream:
        self.url = self._url + self.id

        return super().execute()



from datetime import datetime

ISO_DATE_WITH_MILLIS_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
ISO_DATE_FORMAT = '%Y-%m-%dT%H:%M:%SZ'


def deserialize(iso_date: str) -> datetime or None:
    if iso_date is None:
        return None

    if '.' in iso_date:
        return datetime.strptime(iso_date, ISO_DATE_WITH_MILLIS_FORMAT)
    else:
        return datetime.strptime(iso_date, ISO_DATE_FORMAT)


def serialize(date_time: datetime = datetime.utcnow()) -> str:
    return date_time.strftime(ISO_DATE_FORMAT)



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.file_descriptor import FileDescriptor, ACL
from media_platform.service.lifecycle import Lifecycle


class Destination(Serializable, Deserializable):
    def __init__(self, path: str = None, directory: str = None, acl: ACL = ACL.public, lifecycle: Lifecycle = None,
                 bucket: str = None):
        self._validate_values(path, directory, acl)

        self.path = path
        self.directory = directory
        self.acl = acl or ACL.public
        self.lifecycle = lifecycle
        self.bucket = bucket

    @classmethod
    def deserialize(cls, data: dict) -> Destination:
        lifecycle_data = data.get('lifecycle')
        lifecycle = Lifecycle.deserialize(lifecycle_data) if lifecycle_data else None

        return cls(data.get('path'), data.get('directory'), data['acl'], lifecycle, data.get('bucket'))

    def serialize(self):
        return {
            'path': self.path,
            'directory': self.directory,
            'acl': self.acl,
            'lifecycle': self.lifecycle.serialize() if self.lifecycle else None,
            'bucket': self.bucket
        }

    @staticmethod
    def _validate_values(path: str, directory: str, acl: ACL):
        if not (path or directory):
            raise ValueError('path or directory must be specified')

        FileDescriptor.acl_validator(acl)

        if path:
            FileDescriptor.path_validator(path)

        if directory:
            FileDescriptor.path_validator(directory)



from __future__ import annotations

from media_platform.service.live_service.close_stream_request import CloseStreamRequest
from media_platform.service.live_service.get_stream_request import GetStreamRequest
from media_platform.service.live_service.open_stream_request import OpenStreamRequest
from media_platform.service.media_platform_service import MediaPlatformService


class LiveService(MediaPlatformService):
    def open_stream_request(self) -> OpenStreamRequest:
        return OpenStreamRequest(self._authenticated_http_client, self._base_url)

    def close_stream_request(self) -> CloseStreamRequest:
        return CloseStreamRequest(self._authenticated_http_client, self._base_url)

    def get_stream_request(self) -> GetStreamRequest:
        return GetStreamRequest(self._authenticated_http_client, self._base_url)



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.callback import Callback
from media_platform.service.file_descriptor import ACL
from media_platform.service.file_service.upload_configuration import UploadConfiguration
from media_platform.service.media_platform_request import MediaPlatformRequest


class UploadConfigurationRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/v3/upload/configuration', UploadConfiguration)
        self.path = None
        self.bucket = None
        self.mime_type = None
        self.acl = None
        self.size = None
        self.callback = None

    def set_path(self, path: str) -> UploadConfigurationRequest:
        self.path = path
        return self

    def set_bucket(self, bucket: str) -> UploadConfigurationRequest:
        self.bucket = bucket
        return self

    def set_mime_type(self, mime_type: str) -> UploadConfigurationRequest:
        self.mime_type = mime_type
        return self

    def set_acl(self, acl: ACL) -> UploadConfigurationRequest:
        self.acl = acl
        return self

    def set_size(self, size: int) -> UploadConfigurationRequest:
        self.size = size
        return self

    def set_callback(self, callback: Callback) -> UploadConfigurationRequest:
        self.callback = callback
        return self

    def execute(self) -> UploadConfiguration:
        return super().execute()

    def validate(self):
        # todo
        super().validate()

    def _params(self) -> dict:
        return {
            'path': self.path,
            'bucket': self.bucket,
            'mimeType': self.mime_type,
            'size': self.size,
            'acl': self.acl,
            'callback': self.callback.serialize() if self.callback else None
        }



from __future__ import annotations

from media_platform.service.file_descriptor import FileDescriptor
from media_platform.job.job_type import JobType
from media_platform.job.result.job_result import JobResult


class CreateArchiveResult(JobResult):
    type = JobType.create_archive

    def __init__(self, code: int = None, message: str = None, file_descriptor: FileDescriptor = None):
        super().__init__(code, message)
        self.file_descriptor = file_descriptor

    @classmethod
    def deserialize(cls, data: dict or None) -> CreateArchiveResult or None:
        if not data:
            return None

        payload = data.get('payload')
        result = JobResult.deserialize(data)
        result.__class__ = CreateArchiveResult
        result.file_descriptor = FileDescriptor.deserialize(payload) if payload else None
        return result

    def serialize(self) -> dict:
        data = super().serialize()
        data['payload'] = self.file_descriptor.serialize() if self.file_descriptor else None
        return data



from __future__ import annotations

from media_platform.lang.serialization import Deserializable


class UploadConfiguration(Deserializable):
    def __init__(self, upload_url: str):
        self.upload_url = upload_url

    @classmethod
    def deserialize(cls, data: dict) -> UploadConfiguration:
        return UploadConfiguration(data['uploadUrl'])



from __future__ import annotations

from media_platform.service.file_descriptor import FileDescriptor
from media_platform.job.job_type import JobType
from media_platform.job.result.job_result import JobResult


class ExtractStoryboardResult(JobResult):
    type = JobType.extract_storyboard

    def __init__(self, code: int = None, message: str = None, file_descriptors: [FileDescriptor] = None):
        super().__init__(code, message)
        self.file_descriptors = file_descriptors or []

    @classmethod
    def deserialize(cls, data: dict or None) -> ExtractStoryboardResult or None:
        if data is None:
            return None

        result = JobResult.deserialize(data)
        result.__class__ = ExtractStoryboardResult

        payload_data = data.get('payload') or []
        result.file_descriptors = [FileDescriptor.deserialize(d) for d in payload_data]

        return result

    def serialize(self) -> dict:
        data = super().serialize()
        data['payload'] = [f.serialize() for f in self.file_descriptors]

        return data



from __future__ import annotations

from media_platform.job.job import Job
from media_platform.lang.serialization import Deserializable
# noinspection PyProtectedMember
from media_platform.job.job_deserializer import _JobDeserializer


class _JobGroupResponse(Deserializable):
    def __init__(self, jobs: [Job]):
        self.jobs = jobs

    @classmethod
    def deserialize(cls, data: dict) -> _JobGroupResponse:
        return _JobGroupResponse(
            [_JobDeserializer.deserialize(item) for item in data]
        )



from media_platform.lang.serialization import Deserializable, Serializable


class StreamStateNotification(Serializable, Deserializable):
    def __init__(self, callback_url: str, custom_payload: dict = None):
        self.callback_url = callback_url
        self.custom_payload = custom_payload or {}

    @classmethod
    def deserialize(cls, data: dict):
        return StreamStateNotification(data.get('notifyUrl') or data['callbackUrl'], data.get('customPayload'))

    def serialize(self) -> dict:
        return {
            'callbackUrl': self.callback_url,
            'customPayload': self.custom_payload
        }



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable


class Clipping(Serializable, Deserializable):
    def __init__(self, start: int = None, duration: int = None, fade_in_duration: int = None,
                 fade_out_duration: int = None, fade_in_offset: int = None, fade_out_offset: int = None):
        self.start = start
        self.duration = duration
        self.fade_in_duration = fade_in_duration
        self.fade_out_duration = fade_out_duration
        self.fade_in_offset = fade_in_offset
        self.fade_out_offset = fade_out_offset

    @classmethod
    def deserialize(cls, data: dict) -> Clipping:
        return cls(data.get('start'), data.get('duration'), data.get('fadeInDuration'), data.get('fadeOutDuration'),
                   data.get('fadeInOffset'), data.get('fadeOutOffset'))

    def serialize(self) -> dict:
        return {
            'start': self.start,
            'duration': self.duration,
            'fadeInDuration': self.fade_in_duration,
            'fadeOutDuration': self.fade_out_duration,
            'fadeInOffset': self.fade_in_offset,
            'fadeOutOffset': self.fade_out_offset
        }

    # todo: Add validate()



from __future__ import annotations

from datetime import datetime

from media_platform.lang import datetime_serialization
from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.live_service.enforced_stream_params import EnforcedStreamParams, \
    StreamParamsOutOfRangeErrorInfo
from media_platform.service.live_service.geo_location import GeoLocation
from media_platform.service.live_service.stream_dvr import StreamDVR
from media_platform.service.live_service.stream_error_code import StreamErrorCode
from media_platform.service.live_service.stream_error_info import StreamErrorInfo
from media_platform.service.live_service.stream_playback import StreamPlayback
from media_platform.service.live_service.stream_protocol import StreamProtocol
from media_platform.service.live_service.stream_publisher_endpoint import StreamPublishEndpoint
from media_platform.service.live_service.stream_state import StreamState
from media_platform.service.live_service.stream_state_notification import StreamStateNotification
from media_platform.service.live_service.stream_type import StreamType


class LiveStream(Serializable, Deserializable):
    def __init__(self, project_id: str, streamer_id: str, stream_id: str, playbacks: [StreamPlayback],
                 publish_endpoint: StreamPublishEndpoint or None, dvr: StreamDVR or None,
                 state: StreamState, duration: int, max_publish_duration: int, date_created: datetime,
                 date_updated: datetime,
                 state_notification: StreamStateNotification or None, stream_type: StreamType, success: bool,
                 error_message: str,
                 connect_timeout_secs: int, reconnect_timeout_secs: int, probe_result: dict, protocol: StreamProtocol,
                 enforced_stream_params: EnforcedStreamParams, publisher_geo: GeoLocation, error_code: StreamErrorCode,
                 error_info: StreamErrorInfo):
        self.stream_type = stream_type
        self.project_id = project_id
        self.streamer_id = streamer_id
        self.id = stream_id
        self.playback = playbacks or []
        self.publish_endpoint = publish_endpoint
        self.dvr = dvr
        self.state = state
        self.duration = duration
        self.max_publish_duration = max_publish_duration
        self.state_notification = state_notification
        self.date_created = date_created or datetime.utcnow()
        self.date_updated = date_updated or datetime.utcnow()
        self.success = success
        self.error_message = error_message
        self.connect_timeout_secs = connect_timeout_secs
        self.reconnect_timeout_secs = reconnect_timeout_secs
        self.probe_result = probe_result
        self.protocol = protocol
        self.enforced_stream_params = enforced_stream_params
        self.publisher_geo = publisher_geo
        self.error_code = error_code
        self.error_info = error_info

    @classmethod
    def deserialize(cls, data: dict) -> LiveStream:
        playback_urls_data = data.get('playbackUrls')
        publish_endpoint_data = data.get('publishEndpoint')
        dvr_data = data.get('dvr')
        state_notification_data = data.get('stateNotification')
        enforced_stream_params_data = data.get('enforcedStreamParams')
        publisher_geo_data = data.get('publisherGeo')

        stream_type = data.get('streamType', StreamType.live)
        project_id = data['projectId']
        streamer_id = data['streamerId']
        stream_id = data['id']
        playback_urls = LiveStream._get_playback_urls(playback_urls_data)
        publish_endpoint = StreamPublishEndpoint.deserialize(publish_endpoint_data) if publish_endpoint_data else None
        dvr = StreamDVR.deserialize(dvr_data) if dvr_data else None
        state = data['state']
        duration = data['duration']
        max_publish_duration = data['maxPublishDuration']
        state_notification = StreamStateNotification.deserialize(state_notification_data) \
            if state_notification_data else None

        date_created = datetime_serialization.deserialize(data['dateCreated'])
        date_updated = datetime_serialization.deserialize(data['dateUpdated'])
        success = data.get('success')
        error_message = data.get('errorMessage')
        connect_timeout_secs = data.get('connectTimeout')
        reconnect_timeout_secs = data.get('reconnectTimeout')
        probe_result = data.get('probeResult')
        protocol = data['protocol']
        enforced_stream_params = EnforcedStreamParams.deserialize(enforced_stream_params_data) \
            if enforced_stream_params_data else None
        publisher_geo = GeoLocation.deserialize(publisher_geo_data) if publisher_geo_data else None
        error_code = data.get('errorCode')
        error_info = cls.deserialize_error_info(error_code, data.get('errorInfo'))

        return cls(project_id, streamer_id, stream_id, playback_urls, publish_endpoint, dvr, state,
                   duration, max_publish_duration, date_created, date_updated, state_notification, stream_type,
                   success, error_message, connect_timeout_secs, reconnect_timeout_secs, probe_result, protocol,
                   enforced_stream_params, publisher_geo, error_code, error_info)

    @classmethod
    def _get_playback_urls(cls, playback_urls_data):
        return [StreamPlayback.deserialize(playback) for playback in
                playback_urls_data] if playback_urls_data else None

    def serialize(self) -> dict:
        return {
            'id': self.id,
            'projectId': self.project_id,
            'streamerId': self.streamer_id,
            'publishEndpoint': self.publish_endpoint.serialize() if self.publish_endpoint else None,
            'playbackUrls': [playback.serialize() for playback in self.playback],
            'dvr': self.dvr.serialize() if self.dvr else None,
            'duration': self.duration,
            'maxPublishDuration': self.max_publish_duration,
            'success': self.success,
            'errorMessage': self.error_message,
            'stateNotification': self.state_notification.serialize() if self.state_notification else None,
            'state': self.state,
            'connectTimeout': self.connect_timeout_secs,
            'reconnectTimeout': self.reconnect_timeout_secs,
            'probeResult': self.probe_result,
            'protocol': self.protocol,
            'enforcedStreamParams': self.enforced_stream_params.serialize() if self.enforced_stream_params else None,
            'publisherGeo': self.publisher_geo.serialize() if self.publisher_geo else None,
            'streamType': self.stream_type,
            'dateCreated': datetime_serialization.serialize(self.date_created),
            'dateUpdated': datetime_serialization.serialize(self.date_updated),
            'errorCode': self.error_code,
            'errorInfo': self.error_info.serialize() if self.error_info else None
        }

    @classmethod
    def deserialize_error_info(cls, error_code: StreamErrorCode, data: dict) -> StreamErrorInfo or None:
        if error_code == StreamErrorCode.stream_params_out_of_range:
            return StreamParamsOutOfRangeErrorInfo.deserialize(data)
        else:
            return None



from __future__ import annotations

from media_platform.job.job import Job
from media_platform.job.job_deserializer import _JobDeserializer
from media_platform.lang.serialization import Deserializable


class JobGroup(Deserializable):
    def __init__(self, group_id: str, jobs: [Job]):
        self.group_id = group_id
        self.jobs = jobs

    @classmethod
    def deserialize(cls, data: dict) -> JobGroup:
        jobs = [_JobDeserializer.deserialize(job) for job in data['jobs']]

        return cls(data['groupId'], jobs)



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_service import MediaPlatformService
from media_platform.service.text_service.convert_font_request import ConvertFontRequest
from media_platform.service.text_service.subset_font_request import SubsetFontRequest


class TextService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient):
        super().__init__(domain, authenticated_http_client)

    def convert_font_request(self) -> ConvertFontRequest:
        return ConvertFontRequest(self._authenticated_http_client, self._base_url)

    def subset_font_request(self) -> SubsetFontRequest:
        return SubsetFontRequest(self._authenticated_http_client, self._base_url)



from __future__ import annotations

from media_platform.job.specification import Specification
from media_platform.lang.serialization import Serializable, Deserializable


class VideoSpecification(Specification):
    def __init__(self, codec: VideoCodec, resolution: Resolution = None, frame_rate: float = None,
                 filters: [VideoFilter] = None, frame_rate_fraction: str = None):
        self.codec = codec
        self.resolution = resolution
        self.frame_rate = frame_rate
        self.filters = filters
        self.frame_rate_fraction = frame_rate_fraction

    @classmethod
    def deserialize(cls, data) -> VideoSpecification:
        codec = VideoCodec.deserialize(data['codec'])

        resolution = Resolution.deserialize(data.get('resolution')) if data.get('resolution') else None

        filters_data = data.get('filters')
        filters = [ImageFilter.deserialize(f) for f in filters_data] if filters_data else None

        return VideoSpecification(codec, resolution, data.get('frameRate'), filters, data.get('frameRateFraction'))

    def serialize(self) -> dict:
        data = {
            'codec': self.codec.serialize(),
            'resolution': self.resolution.serialize() if self.resolution else None,
            'filters': [f.serialize() for f in self.filters] if self.filters else None,
        }

        if self.frame_rate_fraction:
            data['frameRateFraction'] = self.frame_rate_fraction
        else:
            data['frameRate'] = self.frame_rate

        return data

    def validate(self):
        if self.resolution:
            self.resolution.validate()


class Resolution(Serializable, Deserializable):
    def __init__(self, width: int = None, height: int = None, scaling: VideoScaling = None,
                 sample_aspect_ratio: str = None):
        self.width = width
        self.height = height
        self.scaling = scaling
        self.sample_aspect_ratio = sample_aspect_ratio  # defaults to '1:1'

    @classmethod
    def deserialize(cls, data: dict) -> Resolution:
        scaling_data = data.get('scaling')
        scaling = VideoScaling.deserialize(scaling_data) if scaling_data else None

        return Resolution(data.get('width'), data.get('height'), scaling, data.get('sampleAspectRatio'))

    def serialize(self) -> dict:
        return {
            'width': self.width,
            'height': self.height,
            'scaling': self.scaling.serialize() if self.scaling else None,
            'sampleAspectRatio': self.sample_aspect_ratio
        }

    def validate(self):
        if self.width and self.width % 2:
            raise ValueError('resolution width %s not divisible by 2' % self.width)

        if self.height and self.height % 2:
            raise ValueError('resolution height %s not divisible by 2' % self.height)


class VideoCodec(Serializable, Deserializable):
    def __init__(self, name: str, profile: str, level: str, crf: int, max_rate: float, gop: GOP = None,
                 preset: str = None):
        self.name = name
        self.profile = profile
        self.level = level
        self.crf = crf
        self.max_rate = max_rate
        self.gop = gop
        self.preset = preset

    @classmethod
    def deserialize(cls, data: dict) -> VideoCodec:
        gop_data = data.get('gop')
        gop = GOP.deserialize(gop_data) if gop_data else None

        return VideoCodec(data['name'], data['profile'], data['level'], data['crf'], data['maxRate'], gop,
                          data.get('preset'))

    def serialize(self) -> dict:
        return {
            'name': self.name,
            'profile': self.profile,
            'level': self.level,
            'crf': self.crf,
            'maxRate': self.max_rate,
            'gop': self.gop.serialize() if self.gop else None,
            'preset': self.preset
        }


class GOP(Serializable, Deserializable):
    def __init__(self, scene_cut: int, key_interval: int, min_key_interval: int, b_frames: int, b_pyramid: int,
                 b_adapt: int, ref_frame: int):
        self.scene_cut = scene_cut
        self.key_interval = key_interval
        self.min_key_interval = min_key_interval
        self.b_frames = b_frames
        self.b_pyramid = b_pyramid
        self.b_adapt = b_adapt
        self.ref_frame = ref_frame

    @classmethod
    def deserialize(cls, data: dict) -> GOP:
        return cls(
            data['sceneCut'],
            data['keyInterval'],
            data['minKeyInterval'],
            data['bFrames'],
            data['bPyramid'],
            data['bAdapt'],
            data['refFrame']
        )

    def serialize(self) -> dict:
        return {
            'sceneCut': self.scene_cut,
            'keyInterval': self.key_interval,
            'minKeyInterval': self.min_key_interval,
            'bFrames': self.b_frames,
            'bPyramid': self.b_pyramid,
            'bAdapt': self.b_adapt,
            'refFrame': self.ref_frame
        }


class VideoScaling(Serializable, Deserializable):
    def __init__(self, algorithm):
        self.algorithm = algorithm

    @classmethod
    def deserialize(cls, data: dict) -> VideoScaling:
        return cls(data['algorithm'])

    def serialize(self) -> dict:
        return {
            'algorithm': self.algorithm
        }


class VideoFilterName:
    unsharp = 'unsharp'
    make_wix_transparent = 'makeWixTransparent'


class VideoFilter(Serializable, Deserializable):
    def __init__(self, name: VideoFilterName, settings: dict = None):
        self.name = name  # 'unsharp'
        self.settings = settings or {}  # '{"value": "5:5:0.5:3:3:0.0"}'

    @classmethod
    def deserialize(cls, data: dict) -> ImageFilter:
        return cls(data['name'], data.get('settings'))

    def serialize(self) -> dict:
        return {
            'name': self.name,
            'settings': self.settings
        }


ImageFilter = VideoFilter
ImageScaling = VideoScaling



from __future__ import annotations

from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.image_service.extract_features_request import ExtractFeaturesRequest
from media_platform.service.image_service.image_operation_request import ImageOperationRequest
from media_platform.service.image_service.image_token import ImageToken, Policy, Watermark
from media_platform.service.media_platform_service import MediaPlatformService


class ImageService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient,
                 app_authenticator: AppAuthenticator):
        super().__init__(domain, authenticated_http_client)
        self.app_authenticator = app_authenticator

    def extract_features_request(self) -> ExtractFeaturesRequest:
        return ExtractFeaturesRequest(self._authenticated_http_client, self._base_url)

    def image_operation_request(self) -> ImageOperationRequest:
        return ImageOperationRequest(self._authenticated_http_client, self._base_url)

    def token(self, policy: Policy = None, watermark: Watermark = None) -> ImageToken:
        token = self.app_authenticator.default_token()

        image_token = ImageToken.from_token(token)
        image_token.policy = policy
        image_token.watermark = watermark

        return image_token

    def sign_token(self, token: ImageToken) -> str:
        return self.app_authenticator.sign_token(token)



from __future__ import annotations

from typing import Type

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.lang.serialization import Deserializable
from media_platform.service.media_platform_request import MediaPlatformRequest


class OrderBy:
    name = 'name'
    date_updated = 'dateUpdated'


class OrderDirection:
    ascending = 'acs'
    descending = 'des'


class _ListRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, url: str,
                 payload_type: Type[Deserializable]):
        super().__init__(authenticated_http_client, 'GET', url, payload_type)
        self.next_page_token = None
        self.page_size = 20
        self.order_by = OrderBy.date_updated
        self.order_direction = OrderDirection.descending

    def set_next_page_token(self, next_page_token: str):
        self.next_page_token = next_page_token
        return self

    def set_page_size(self, page_size: int):
        self.page_size = page_size
        return self

    def set_order_by(self, order_by: OrderBy):
        self.order_by = order_by
        return self

    def set_order_direction(self, order_direction: OrderDirection):
        self.order_direction = order_direction
        return self

    def _params(self) -> dict:
        params = {}

        if self.next_page_token:
            params['nextPageToken'] = self.next_page_token

        if self.page_size:
            params['pageSize'] = self.page_size

        if self.order_by:
            params['orderBy'] = self.order_by

        if self.order_direction:
            params['orderDirection'] = self.order_direction

        return params



from __future__ import annotations

from media_platform.service.file_descriptor import FileDescriptor
from media_platform.job.job_type import JobType
from media_platform.job.result.job_result import JobResult


class TranscodeResult(JobResult):
    type = JobType.transcode

    def __init__(self, code=None, message=None, file_descriptor=None, master_ffmpeg_command=None):
        super().__init__(code, message)
        self.file_descriptor = file_descriptor
        self.master_ffmpeg_command = master_ffmpeg_command

    @classmethod
    def deserialize(cls, data: dict or None) -> TranscodeResult or None:
        if data is None:
            return None

        result = JobResult.deserialize(data)
        result.__class__ = TranscodeResult

        result.file_descriptor = None
        result.master_ffmpeg_command = None

        payload_data = data.get('payload')
        if payload_data:
            # todo: payload.file is deprecated
            file_descriptor_data = payload_data.get('file')
            if file_descriptor_data:
                result.file_descriptor = FileDescriptor.deserialize(file_descriptor_data)

            result.master_ffmpeg_command = payload_data.get('masterFFMpegCommand')

        return result

    def serialize(self) -> dict:
        data = super(TranscodeResult, self).serialize()
        data['payload'] = self._serialize_payload()

        return data

    def _serialize_payload(self) -> dict:
        payload = None

        if self.file_descriptor:
            payload = self.file_descriptor.serialize()
            # backwards compatibility
            payload['file'] = self.file_descriptor.serialize()

        if self.master_ffmpeg_command:
            if not payload:
                payload = {}

            payload['masterFFMpegCommand'] = self.master_ffmpeg_command

        return payload



from __future__ import annotations

from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.job.job import Job
from media_platform.service.file_service.external_authorization import ExternalAuthorization


class ImportFileSpecification(Specification):
    def __init__(self, source_url: str, destination: Destination, external_authorization: ExternalAuthorization = None):
        self.source_url = source_url
        self.destination = destination
        self.external_authorization = external_authorization

    def serialize(self) -> dict:
        return {
            'sourceUrl': self.source_url,
            'destination': self.destination.serialize(),
            'externalAuthorization': self.external_authorization.serialize() if self.external_authorization else None
        }

    @classmethod
    def deserialize(cls, data: dict) -> ImportFileSpecification:
        destination = Destination.deserialize(data['destination'])

        external_authorization_data = data.get('externalAuthorization')
        external_authorization = None
        if external_authorization_data:
            external_authorization = ExternalAuthorization.deserialize(external_authorization_data)

        return ImportFileSpecification(data['sourceUrl'], destination, external_authorization)


class ImportFileJob(Job):
    type = JobType.import_file
    specification_type = ImportFileSpecification



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable


class Callback(Serializable, Deserializable):
    def __init__(self, url: str, attachment: dict = None, headers: dict = None, passthrough: bool = False):
        self.url = url
        self.attachment = attachment
        self.headers = headers
        self.passthrough = passthrough

    @classmethod
    def deserialize(cls, data: dict) -> Callback:
        return cls(data['url'], data.get('attachment'), data.get('headers'), data.get('passthrough', False))

    def serialize(self) -> dict:
        return {
            'url': self.url,
            'attachment': self.attachment,
            'headers': self.headers,
            'passthrough': self.passthrough
        }



from setuptools import setup, find_packages

PACKAGE_NAME = 'wix_media_platform'
PACKAGE_VERSION = '1.42.0'

setup(
    name=PACKAGE_NAME,
    version=PACKAGE_VERSION,
    description='Wix Media Platform python SDK',
    author='Wix Media Platform',
    author_email='mcloud@wix.com',
    url='https://github.com/wix/media-platform-python-sdk',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    packages=find_packages(include=('media_platform', 'media_platform.*',)),
    install_requires=[
        'python-jose[cryptography]',
        'requests',
        'requests-toolbelt',
        'furl',
    ],
    tests_require=[
        'PyHamcrest',
        'mockito',
        'httpretty'
    ]
)



from __future__ import annotations

from typing import Set

from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.callback import Callback
from media_platform.service.flow_control_service.operation import OperationStatus
from media_platform.service.source import Source


class ErrorStrategy:
    stop_on_error = 'stopOnError'
    continue_on_error = 'continueOnError'


class Invocation(Serializable, Deserializable):
    def __init__(self, entry_points: [str], sources: [Source] = None, callback: Callback = None,
                 error_strategy: ErrorStrategy = None, operation_callback_status_filter: Set[OperationStatus] = None):
        self.entry_points = entry_points
        self.sources = sources or []
        self.callback = callback
        self.error_strategy = error_strategy or ErrorStrategy.stop_on_error
        self.operation_callback_status_filter = operation_callback_status_filter or set()

    def serialize(self) -> dict:
        return {
            'entryPoints': self.entry_points,
            'sources': [source.serialize() for source in self.sources],
            'callback': self.callback.serialize() if self.callback else None,
            'errorStrategy': self.error_strategy,
            'operationCallbackStatusFilter': list(self.operation_callback_status_filter)
        }

    @classmethod
    def deserialize(cls, data: dict) -> Invocation:
        sources = [Source.deserialize(s) for s in data['sources']]
        callback_data = data.get('callback')
        callback = Callback.deserialize(callback_data) if callback_data else None

        return cls(data['entryPoints'], sources, callback, data.get('errorStrategy'),
                   set(data.get('operationCallbackStatusFilter', [])))



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_request import MediaPlatformRequest


class AbortFlowRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'DELETE', base_url + '/flow_control/flow/', None)
        self.flow_id = None
        self._url = base_url + '/flow_control/flow/'

    def set_id(self, flow_id: str) -> AbortFlowRequest:
        self.flow_id = flow_id
        return self

    def execute(self):
        self.url = self._url + self.flow_id

        return super().execute()



from __future__ import annotations

from media_platform.lang.serialization import Deserializable


class Label(Deserializable):
    def __init__(self, name: str, score: float):
        self.name = name
        self.score = score

    @classmethod
    def deserialize(cls, data: dict) -> Label:
        return Label(data['name'], data['score'])



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_service import MediaPlatformService
from media_platform.service.video_service.extract_poster_request import ExtractPosterRequest
from media_platform.service.video_service.extract_storyboard_request import ExtractStoryboardRequest


class VideoService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient):
        super().__init__(domain, authenticated_http_client)

    def extract_poster_request(self) -> ExtractPosterRequest:
        return ExtractPosterRequest(self._authenticated_http_client, self._base_url)

    def extract_storyboard_request(self) -> ExtractStoryboardRequest:
        return ExtractStoryboardRequest(self._authenticated_http_client, self._base_url)



from __future__ import annotations

from media_platform.lang.serialization import Serializable, Deserializable


class GeoLocation(Serializable, Deserializable):
    def __init__(self, latitude: float = None, longitude: float = None, ip_address: str = None, country: str = None,
                 city: str = None):
        self.latitude = latitude
        self.longitude = longitude
        self.ip_address = ip_address
        self.country = country
        self.city = city

    def serialize(self) -> dict:
        return {
            'coordinates': {
                'latitude': self.latitude,
                'longitude': self.longitude
            },
            'ipAddress': self.ip_address,
            'country': self.country,
            'city': self.city
        }

    @classmethod
    def deserialize(cls, data: dict) -> GeoLocation:
        coordinates_data = data.get('coordinates') or {}
        return cls(coordinates_data.get('latitude'),
                   coordinates_data.get('longitude'),
                   data.get('ipAddress'),
                   data.get('country'),
                   data.get('city'))



from __future__ import annotations

from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.archive_service.archive_manifest_url_request import ArchiveManifestUrlRequest
from media_platform.service.archive_service.create_archive_manifest_request import CreateArchiveManifestRequest
from media_platform.service.archive_service.create_archive_request import CreateArchiveRequest
from media_platform.service.archive_service.extract_archive_request import ExtractArchiveRequest
from media_platform.service.media_platform_service import MediaPlatformService


class ArchiveService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient, app_id: str,
                 app_authenticator: AppAuthenticator):
        super().__init__(domain, authenticated_http_client)
        self._app_id = app_id
        self._app_authenticator = app_authenticator

    def create_archive_request(self) -> CreateArchiveRequest:
        return CreateArchiveRequest(self._authenticated_http_client, self._base_url)

    def create_archive_manifest_request(self) -> CreateArchiveManifestRequest:
        return CreateArchiveManifestRequest(self._authenticated_http_client, self._base_url)

    def archive_manifest_url_request(self) -> ArchiveManifestUrlRequest:
        return ArchiveManifestUrlRequest(self._app_id, self._app_authenticator, self._domain)

    def extract_archive_request(self) -> ExtractArchiveRequest:
        return ExtractArchiveRequest(self._authenticated_http_client, self._base_url)



from __future__ import annotations

from media_platform.lang.serialization import Deserializable
from media_platform.metadata.features.crop_hint import CropHint
from media_platform.metadata.features.explicit_content import ExplicitContent
from media_platform.metadata.features.image_color import Color
from media_platform.metadata.features.label import Label
from media_platform.metadata.features.rectangle import Rectangle


class ImageFeatures(Deserializable):
    def __init__(self, labels: [Label] = None, faces: [Rectangle] = None, colors: [Color] = None,
                 explicit_content: [ExplicitContent] = None, crop_hints: [CropHint] = None):
        self.labels = labels or []
        self.faces = faces or []
        self.colors = colors or []
        self.explicit_content = explicit_content or []
        self.crop_hints = crop_hints or []

    @classmethod
    def deserialize(cls, data: dict) -> ImageFeatures:
        labels = [Label.deserialize(label) for label in data.get('labels', [])]
        faces = [Rectangle.deserialize(face) for face in data.get('faces', [])]
        colors = [Color.deserialize(color) for color in data.get('colors', [])]
        explicit_content = [ExplicitContent.deserialize(content) for content in data.get('explicitContent', [])]
        crop_hints = [CropHint.deserialize(content) for content in data.get('cropHints', [])]

        return ImageFeatures(labels, faces, colors, explicit_content, crop_hints)



from __future__ import annotations

from media_platform.lang.serialization import Deserializable
from media_platform.metadata.audio_file_metadata import AudioFileMetadata
from media_platform.metadata.file_metadata import FileMetadata, MediaType
from media_platform.metadata.font_file_metadata import FontFileMetadata
from media_platform.metadata.image_file_metadata import ImageFileMetadata
from media_platform.metadata.video_file_metadata import VideoFileMetadata


class _FileMetadataDeserializer(Deserializable):

    @classmethod
    def deserialize(cls, data: dict) -> FileMetadata:
        media_type = data['mediaType']

        if media_type == MediaType.video:
            return VideoFileMetadata.deserialize(data)
        elif media_type == MediaType.image:
            return ImageFileMetadata.deserialize(data)
        elif media_type == MediaType.audio:
            return AudioFileMetadata.deserialize(data)
        elif media_type == MediaType.font:
            return FontFileMetadata.deserialize(data)
        else:
            return FileMetadata.deserialize(data)



from abc import ABC, abstractmethod

from media_platform.lang.serialization import Serializable, Deserializable


class Specification(Serializable, Deserializable, ABC):

    @abstractmethod
    def serialize(self) -> dict:
        pass

    @classmethod
    @abstractmethod
    def deserialize(cls, data):
        pass

    # override for request pre-flight check
    def validate(self):
        pass



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.create_archive_job import ArchiveSource, CreateArchiveJob, ArchiveType
from media_platform.service.callback import Callback
from media_platform.service.destination import Destination
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class CreateArchiveRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/archive/create', CreateArchiveJob)
        self.sources = []
        self.destination = None
        self.archive_type = 'zip'
        self.callback = None

    def set_sources(self, sources: [Source or ArchiveSource]) -> CreateArchiveRequest:
        self.sources = sources
        return self

    def add_sources(self, *sources: [Source or ArchiveSource]) -> CreateArchiveRequest:
        self.sources.extend(sources)
        return self

    def set_destination(self, destination: Destination) -> CreateArchiveRequest:
        self.destination = destination
        return self

    def set_archive_type(self, archive_type: ArchiveType) -> CreateArchiveRequest:
        self.archive_type = archive_type
        return self

    def set_callback(self, callback: Callback) -> CreateArchiveRequest:
        self.callback = callback
        return self

    def execute(self) -> CreateArchiveJob:
        return super().execute()

    def _params(self) -> dict:
        return {
            'sources': [source.serialize() for source in self.sources if source],
            'destination': self.destination.serialize(),
            'archiveType': self.archive_type,
            'jobCallback': self.callback.serialize() if self.callback else None
        }



from __future__ import annotations

from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification
from media_platform.job.transcode.video_specification import Resolution
from media_platform.service.destination import Destination
from media_platform.job.job import Job


class PosterImageFormat:
    jpeg = 'jpg'
    png = 'png'
    values = [jpeg, png]
    invalid_value_message = 'Image format must be one of: %s' % ', '.join(values)


class PosterFilter:
    video = 'video'
    wix_alpha = 'wixAlpha'
    video_alpha = 'videoAlpha'

    # backwards compatibility - same as wixAlpha
    transparent_crop = 'transparentCrop'

    values = [video, wix_alpha, video_alpha, transparent_crop]
    invalid_value_message = 'Filters must be one of: %s' % ', '.join(values)


class PixelFormat:
    rgb24 = 'rgb24'
    rgba = 'rgba'

    values = [None, rgb24, rgba]

    invalid_value_message = 'Pixel format must be one of: %s' % ', '.join(str(v) for v in values)


class ExtractPosterSpecification(Specification):
    def __init__(self, second: float = None, destination: Destination = None,
                 image_format: PosterImageFormat = PosterImageFormat.jpeg, percentage: float = None,
                 filters: [PosterFilter] = None, resolution: Resolution = None, pixel_format: PixelFormat = None):
        self.second = second
        self.destination = destination
        self.image_format = image_format or PosterImageFormat.jpeg
        self.percentage = percentage
        self.filters = filters or []
        self.resolution = resolution
        self.pixel_format = pixel_format

    def serialize(self) -> dict:
        data = {
            'second': self.second,
            'percentage': self.percentage,
            'destination': self.destination.serialize(),
            'format': self.image_format,
            'filters': self.filters,
        }

        if self.resolution:
            data['resolution'] = self.resolution.serialize()

        if self.pixel_format:
            data['pixelFormat'] = self.pixel_format

        return data

    @classmethod
    def deserialize(cls, data: dict) -> ExtractPosterSpecification:
        destination = Destination.deserialize(data['destination'])

        resolution_data = data.get('resolution')
        resolution = Resolution.deserialize(resolution_data) if resolution_data else None
        return ExtractPosterSpecification(data['second'], destination, data['format'], data.get('percentage'),
                                          data.get('filters'), resolution, data.get('pixelFormat'))

    def validate(self):
        self._validate_image_format()
        self._validate_time()
        self._validate_filters()

    def _validate_image_format(self):
        if self.image_format not in PosterImageFormat.values:
            raise ValueError(PosterImageFormat.invalid_value_message)

    def _validate_time(self):
        if self.second is None and self.percentage is None:
            raise ValueError('Must provide either second or percentage')

        if self.percentage is not None and (self.percentage < 0 or self.percentage > 100):
            raise ValueError('Percentage must be 0-100')

    def _validate_filters(self):
        invalid_filters = [f for f in self.filters if f not in PosterFilter.values]
        if invalid_filters:
            raise ValueError(PosterFilter.invalid_value_message)


class ExtractPosterJob(Job):
    type = JobType.extract_poster
    specification_type = ExtractPosterSpecification



from __future__ import annotations

from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor


class _CreateFilesResponse(Deserializable):
    def __init__(self, file_descriptors: [FileDescriptor]):
        self.file_descriptors = file_descriptors

    @classmethod
    def deserialize(cls, data: dict) -> _CreateFilesResponse:
        return cls([FileDescriptor.deserialize(f) for f in data['files']])



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.extract_poster_job_group import ExtractPosterJobGroup
from media_platform.job.extract_storyboard_job import ExtractStoryboardSpecification
from media_platform.job.extract_storyboard_job_group import ExtractStoryboardJobGroup
from media_platform.service.callback import Callback
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class ExtractStoryboardRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', base_url + '/av/storyboard', ExtractStoryboardJobGroup)
        self.sources = []
        self.specifications = []
        self.callback = None

    def set_sources(self, sources: [Source]):
        self.sources = sources
        return self

    def add_sources(self, *sources: [Source]):
        self.sources.extend(sources)
        return self

    def set_specifications(self, specifications: [ExtractStoryboardSpecification]):
        self.specifications = specifications
        return self

    def add_specifications(self, *specifications: [ExtractStoryboardSpecification]):
        self.specifications.extend(specifications)
        return self

    def set_callback(self, callback: Callback):
        self.callback = callback
        return self

    def validate(self):
        [specification.validate() for specification in self.specifications]

    def execute(self) -> ExtractPosterJobGroup:
        return super().execute()

    def _params(self) -> dict:
        return {
            'sources': [source.serialize() for source in self.sources],
            'specifications': [specification.serialize() for specification in self.specifications],
            'jobCallback': self.callback.serialize() if self.callback else None
        }



from __future__ import annotations

from media_platform.service.destination import Destination
from typing import Dict

from media_platform.lang.serialization import Deserializable, Serializable


class StreamDVR(Deserializable, Serializable):
    def __init__(self, destination: Destination):
        self.destination = destination

    @classmethod
    def deserialize(cls, data: dict) -> StreamDVR:
        destination = Destination.deserialize(data['destination'])

        return cls(destination)

    def serialize(self) -> Dict:
        return {
            'destination': self.destination.serialize()
        }
